r"""
Installation requirements:
    pip install trame trame-vuetify trame-components
"""

from collections import defaultdict
from pathlib import Path

from trame.app import get_server
from trame.assets.local import LocalFileManager
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import trame

BASE = Path(__file__).parent

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Application
# -----------------------------------------------------------------------------


class PipelineManager:
    DEFAULT_NODE = {"name": "undefined", "visible": 0, "collapsed": 0}

    def __init__(self, state, name):
        self._state = state
        self._name = name
        self._next_id = 1
        self._nodes = {}
        self._children_map = defaultdict(set)

    def _update_hierarchy(self):
        self._children_map.clear()
        for node in self._nodes.values():
            self._children_map[node.get("parent")].add(node.get("id"))

        self.update()

    def _add_children(self, list_to_fill, node_id):
        for child_id in self._children_map[node_id]:
            node = self._nodes[child_id]
            list_to_fill.append(node)
            if node.get("collapsed"):
                continue
            self._add_children(list_to_fill, node.get("id"))

        return list_to_fill

    def update(self):
        result = self._add_children([], "0")
        self._state[self._name] = result
        return result

    def add_node(self, parent="0", **item_keys):
        _id = f"{self._next_id}"
        self._next_id += 1

        node = {
            **PipelineManager.DEFAULT_NODE,
            "id": _id,
            "parent": parent,
            **item_keys,
        }
        self._nodes[_id] = node
        self._update_hierarchy()

        return _id

    def remove_node(self, _id):
        for id in self._children_map[_id]:
            self.remove_node(id)
        self._nodes.pop(_id)
        self._update_hierarchy()

    def toggle_collapsed(self, _id, icons=["collapsed", "collapsible"]):
        node = self.get_node(_id)
        node["collapsed"] = not node["collapsed"]

        # Toggle matching action icon
        actions = node.get("actions", [])
        for i in range(len(actions)):
            action = actions[i]
            if action in icons:
                actions[i] = icons[(icons.index(action) + 1) % 2]

        self.update()
        return node["collapsed"]

    def toggle_visible(self, _id):
        node = self.get_node(_id)
        node["visible"] = not node["visible"]
        self.update()
        return node["visible"]

    def get_node(self, _id):
        return self._nodes.get(f"{_id}")


pipeline = PipelineManager(state, "git_tree")

id_root = pipeline.add_node(
    name="root", visible=0, color="#9C27B0", actions=["test", "delete"]
)
id_a = pipeline.add_node(
    parent=id_root,
    name="a",
    visible=1,
    color="#42A5F5",
    actions=["collapsible", "delete"],
)
id_b = pipeline.add_node(
    parent=id_root, name="b", visible=1, color="#00ACC1", actions=["collapsible"]
)
id_aa = pipeline.add_node(
    parent=id_a, name="aa", visible=1, color="#2962FF", actions=["test", "delete"]
)
id_aaa = pipeline.add_node(parent=id_aa, name="aaa", visible=1, color="black")
id_ba = pipeline.add_node(
    parent=id_b, name="ba", visible=1, color="#004D40", actions=["collapsible"]
)
id_bb = pipeline.add_node(
    parent=id_b, name="bb", visible=1, color="#80CBC4", actions=["collapsible"]
)
id_bba = pipeline.add_node(parent=id_bb, name="bba", visible=1, color="#00838F")
id_bbb = pipeline.add_node(parent=id_bb, name="bbb", visible=1, color="#4DB6AC")

pipeline.update()

local_file_manager = LocalFileManager(__file__)
local_file_manager.url("test", BASE / "icons/abacus.svg")
local_file_manager.url("delete", BASE / "icons/trash-can-outline.svg")
local_file_manager.url("collapsed", BASE / "icons/chevron-up.svg")
local_file_manager.url("collapsible", BASE / "icons/chevron-down.svg")

# -----------------------------------------------------------------------------
# Callback
# -----------------------------------------------------------------------------

# @state.change("git_tree")
# def pipeline_changed(git_tree, **kwargs):
#     print(git_tree)


def on_action(event):
    print("on_action", event)
    _id = event.get("id")
    _action = event.get("action")
    if _action.startswith("collap"):
        print(pipeline.toggle_collapsed(_id))


def on_event(event):
    print(event)


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.title.set_text("Git Tree")

    with layout.content:
        trame.GitTree(
            sources=("git_tree",),
            action_map=("icons", local_file_manager.assets),
            action_size=25,
            action=(on_action, "[$event]"),
            # visibility_change=(on_event, "[$event]"),
            # actives_change=(on_event, "[$event]"),
        )

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout, VAppLayout
from trame.widgets import vuetify3


class MultiWindow:
    def __init__(self, server=None, table_size=10):
        self.server = get_server(server, client_type="vue3")

        # Create local variable for JS side
        self.state.update(dict(window_hello=None, window_world=None))
        self.state.client_only("window_hello", "window_world")

        # Force ui creation
        self.ui = self.ui_main
        self.popup_hello = self.ui_hello
        self.popup_world = self.ui_world

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    @property
    def ui_main(self):
        with SinglePageLayout(self.server, full_height=True) as layout:
            with layout.toolbar.clear():
                vuetify3.VToolbarTitle("Multi Window example")
                vuetify3.VSpacer()
                vuetify3.VBtn(
                    "Open Hello",
                    disabled=("window_hello",),
                    click="""
                        window_hello = window.open(
                            '/?ui=hello',
                            target='_blank',
                            'popup,width=200,height=200,left=10,top=10'
                        )
                    """,
                )
                vuetify3.VBtn(
                    "Close Hello", click="window_hello.close(); window_hello = null;"
                )
                vuetify3.VBtn(
                    "Open World",
                    disabled=("window_world",),
                    click="""
                        window_world = window.open(
                            '/?ui=world',
                            target='_blank',
                            'popup,width=200,height=200,left=100,top=100'
                        )
                    """,
                )
                vuetify3.VBtn(
                    "Close World", click="window_world.close(); window_world = null;"
                )
            with layout.content:
                with vuetify3.VContainer():
                    with vuetify3.VCard():
                        vuetify3.VCardTitle("Main Page")

    @property
    def ui_hello(self):
        with VAppLayout(self.server, template_name="hello", full_height=True):
            with vuetify3.VContainer():
                with vuetify3.VCard():
                    vuetify3.VCardTitle("Hello Page")

    @property
    def ui_world(self):
        with VAppLayout(self.server, template_name="world", full_height=True):
            with vuetify3.VContainer():
                with vuetify3.VCard():
                    vuetify3.VCardTitle("World Page")


def main():
    app = MultiWindow()
    app.server.start()


if __name__ == "__main__":
    main()



from pathlib import Path

from trame.app import get_server
from trame.ui.vuetify import VAppLayout

server = get_server(client_type="vue2")

SCRIPT_PATH = Path(__file__).with_name("291.js")
split_pane_module = {
    "serve": {
        "__trame_splitpanes": str(SCRIPT_PATH.parent.absolute()),
    },
    "scripts": ["https://unpkg.com/splitpanes@legacy", "__trame_splitpanes/291.js"],
    "styles": ["https://unpkg.com/splitpanes@legacy/dist/splitpanes.css"],
    "vue_use": ["trame_splitpanes"],
}
server.enable_module(split_pane_module)

with VAppLayout(server) as layout:
    layout.root.add_child(
        """
    <splitpanes class="default-theme">
        <pane min-size="20">
            1
            <br>
            <em class="specs"> I have a min width of 20%<em>
        </pane>
        <pane>
            <splitpanes class="default-theme" horizontal>
                <pane min-size="15">
                    2
                    <br>
                    <em class="specs"> I have a min height of 15%</em>
                </pane>
                <pane>3</pane>
                <pane>4</pane>
            </splitpanes>
        </pane>
        <pane>
            5
        </pane>
    </splitpanes>
    """
    )

server.start()



import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersSources import vtkConeSource

# VTK factory initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import TrameApp
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import html, vtk, vuetify3
from trame.decorators import change


class Cone(TrameApp):
    def __init__(self, name=None):
        super().__init__(server=name)
        self._setup_vtk()
        self._build_ui()

    def _setup_vtk(self):
        renderer = vtkRenderer()
        render_window = vtkRenderWindow()
        render_window.AddRenderer(renderer)
        render_window.OffScreenRenderingOn()

        render_window_interactor = vtkRenderWindowInteractor()
        render_window_interactor.SetRenderWindow(render_window)
        render_window_interactor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

        cone_source = vtkConeSource()
        mapper = vtkPolyDataMapper()
        actor = vtkActor()
        mapper.SetInputConnection(cone_source.GetOutputPort())
        actor.SetMapper(mapper)
        renderer.AddActor(actor)
        renderer.ResetCamera()
        render_window.Render()

        self.cone_source = cone_source
        self.render_window = render_window

    @property
    def resolution(self):
        return self.state.resolution

    @resolution.setter
    def resolution(self, v):
        with self.state:
            self.state.resolution = v

    def reset_resolution(self):
        self.resolution = 6

    def _build_ui(self):
        with SinglePageLayout(self.server) as self.ui:
            self.ui.icon.click = self.ctrl.view_reset_camera
            with self.ui.toolbar:
                html.Div("resolution: {{ resolution }}")
                vuetify3.VSpacer()
                vuetify3.VSlider(
                    v_model=("resolution", 6), min=3, max=60, step=1, hide_details=True
                )
                vuetify3.VBtn("Reset", click=self.reset_resolution)
            with self.ui.content:
                with vtk.VtkRemoteView(
                    self.render_window,
                    interactive_ratio=1,
                ) as view:
                    self.ctrl.view_update = view.update
                    self.ctrl.view_reset_camera = view.reset_camera

    @change("resolution")
    def _on_resolution(self, resolution, **_):
        self.cone_source.SetResolution(resolution)
        self.ctrl.view_update()


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    app = Cone()
    app.server.start()



r"""
https://v2.vuetifyjs.com/en/components/tooltips/

    <v-tooltip bottom>
      <template v-slot:activator="{ on, attrs }">
        <v-btn v-bind="attrs" v-on="on">Hover over me</v-btn>
      </template>
      <span>Tooltip</span>
    </v-tooltip>
"""

from trame.app import get_server
from trame.ui.vuetify2 import VAppLayout
from trame.widgets import html, vuetify2

server = get_server(client_type="vue2")

with VAppLayout(server):
    with vuetify2.VTooltip(bottom=True):
        with vuetify2.Template(v_slot_activator="{ on, attrs }"):
            vuetify2.VBtn(
                "Hover over me",
                v_bind="attrs",
                v_on="on",
            )
        html.Span("Tooltip")

server.start(port=10002)



import asyncio

# Required for rendering initialization, not necessary for
# local rendering, but doesn't hurt to include it
import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkCommonColor import vtkNamedColors
from vtkmodules.vtkFiltersSources import vtkConeSource, vtkSphereSource

# Required for interactor initialization
from vtkmodules.vtkInteractionStyle import (
    vtkInteractorStyleSwitch,  # noqa
    vtkInteractorStyleTrackballCamera,
)
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import asynchronous, get_server
from trame.ui.vuetify import VAppLayout
from trame.widgets import vtk, vuetify

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------


colors = vtkNamedColors()


def cone():
    cone = vtkConeSource()
    cone.SetResolution(60)

    # Create a mapper and actor
    mapper = vtkPolyDataMapper()
    mapper.SetInputConnection(cone.GetOutputPort())

    actor = vtkActor()
    actor.SetMapper(mapper)
    actor.GetProperty().SetDiffuseColor(colors.GetColor3d("bisque"))

    # Visualize
    renderer = vtkRenderer()
    renderWindow = vtkRenderWindow()
    renderWindow.AddRenderer(renderer)
    renderWindowInteractor = vtkRenderWindowInteractor()
    interactor_style = vtkInteractorStyleTrackballCamera()
    renderWindowInteractor.SetInteractorStyle(interactor_style)
    renderWindowInteractor.SetRenderWindow(renderWindow)

    renderer.AddActor(actor)
    renderer.SetBackground(colors.GetColor3d("bisque"))
    renderer.ResetCamera()

    renderWindow.SetSize(640, 480)
    renderWindow.SetWindowName("Cone")

    return renderWindow


def sphere():
    sphere = vtkSphereSource()

    # Create a mapper and actor
    mapper = vtkPolyDataMapper()
    mapper.SetInputConnection(sphere.GetOutputPort())

    actor = vtkActor()
    actor.SetMapper(mapper)
    actor.GetProperty().SetDiffuseColor(colors.GetColor3d("bisque"))

    # Visualize
    renderer = vtkRenderer()
    renderWindow = vtkRenderWindow()
    renderWindow.AddRenderer(renderer)
    renderWindowInteractor = vtkRenderWindowInteractor()
    interactor_style = vtkInteractorStyleTrackballCamera()
    renderWindowInteractor.SetInteractorStyle(interactor_style)
    renderWindowInteractor.SetRenderWindow(renderWindow)

    renderer.AddActor(actor)
    renderer.SetBackground(colors.GetColor3d("Salmon"))
    renderer.ResetCamera()

    renderWindow.SetSize(640, 480)
    renderWindow.SetWindowName("Sphere")

    return renderWindow


cone_window = cone()
sphere_window = sphere()

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# Initial VTK window width and height values
state.vtk_window_width = 300
state.vtk_window_height = 300


# -----------------------------------------------------------------------------
# Background thread
# -----------------------------------------------------------------------------


@asynchronous.task
async def refresh_function(**kwargs):
    counter = 1
    while True:
        with state:
            if counter % 2 == 0:
                ren_win = sphere_window
            else:
                ren_win = cone_window

            ctrl.view_replace(ren_win)
            ctrl.view_update()

            counter += 1

        await asyncio.sleep(5)


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------


with VAppLayout(server) as layout:
    with layout.root:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            view = vtk.VtkRemoteView(cone_window)
            # view = vtk.VtkLocalView(cone_window)
            ctrl.view_update = view.update
            ctrl.view_replace = view.replace_view
            ctrl.on_server_ready.add(refresh_function)


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame_common.exec.asynchronous import create_task, decorate_task, task
from trame_server.utils.asynchronous import (
    StateQueue,
    create_state_queue_monitor_task,
)

__all__ = [
    "create_task",
    "decorate_task",
    "task",
    # state sync across threads
    "StateQueue",
    "create_state_queue_monitor_task",
]



import sys

from trame_server import Server

from trame.app import dev, get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


def reset_resolution():
    state.resolution = 6


# Mode A ----------------------------------------------------------------------
def full_reload(server: Server):
    print("=> Reload mode A")
    dev.remove_change_listeners(server, "resolution")
    dev.reload(sys.modules.get("__main__"))


# Mode B ----------------------------------------------------------------------
def reload_app(server: Server):
    print("=> Reload mode B")
    # dev.clear_triggers(app) # Not needed here
    dev.clear_change_listeners(server)
    dev.reload(sys.modules.get("__main__"))


# Reload mode selector --------------------------------------------------------
ctrl.on_server_reload = ctrl.on_server_reload.clear  # Will happen first

if True:
    ctrl.on_server_reload.add(reload_app)
else:
    ctrl.on_server_reload.add(full_reload)


@state.change("resolution")
def update_resolution(resolution, **kwargs):
    print("Update resolution", resolution)


# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

layout = SinglePageLayout(server)

with layout:
    layout.title.set_text("Dynamic reload")
    layout.toolbar.dense = True

    # Toolbar
    with layout.toolbar as toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            # label="hello", # comment/uncomment between reload
            hide_details=True,
            v_model=("resolution", 6),
            max=60,
            min=3,
            step=1,
            style="max-width: 300px;",
        )
        vuetify.VSwitch(
            hide_details=True,
            v_model=("$vuetify.theme.dark",),
        )
        with vuetify.VBtn(icon=True, click=reset_resolution):
            vuetify.VIcon("mdi-undo")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            with vtk.VtkView() as view:
                ctrl.reset_camera = view.reset_camera
                with vtk.VtkGeometryRepresentation():
                    vtk.VtkAlgorithm(
                        vtk_class="vtkConeSource", state=("{ resolution }",)
                    )


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__" and not server.running:
    server.start()



def test_TrameApp():
    import asyncio

    from trame.app import get_server
    from trame.decorators import TrameApp, change, controller

    @TrameApp()
    class A:
        def __init__(self, server, e):
            """doc string"""
            self.server = server
            self.e = e

        @change("c")
        def c_changed(self, c, **kwargs):
            self.c = c

    class B(A):
        def __init__(self, server, e):
            super().__init__(server, e)

        @change("d")
        def d_changed(self, d, **kwargs):
            self.d = d

        @controller.set("on_server_ready")
        def on_server_ready(self, **kwargs):
            assert self.c == 10
            assert self.d == 15
            asyncio.create_task(self.server.stop())

    server = get_server()

    a = A(server, 1)
    assert type(a) is A
    assert a.__init__.__doc__ is not None

    b = B(server, 2)
    assert type(b) is B
    assert b.e == 2

    b.server.state.c = 10
    b.server.state.d = 15
    b.server.start()



from IPython.display import HTML, display_html

display_html(
    HTML("""
        <style>
        .jp-LinkedOutputView .jp-OutputArea-child:has(iframe[id*='trame__template']) {
        height: 100%;
        }
        </style>
        """)
)


def fullscreen(app_or_ui):
    original = app_or_ui
    if hasattr(app_or_ui, "ui"):
        app_or_ui = app_or_ui.ui

    if hasattr(app_or_ui, "iframe_style"):
        app_or_ui.iframe_style = "border: none; width: 100%; height: 100%;"

    return original



import asyncio
import sys

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import trame, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


def kill_server():
    sys.exit(0)


def throw_exception():
    raise Exception("That is an expected error")


async def fake_busy():
    print("start busy...")
    asyncio.sleep(5)
    print("end busy...")


ctrl.on_client_mounted.add(fake_busy)

# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

layout = SinglePageLayout(server)

with layout:
    # Validate client life cycle
    trame.LifeCycleMonitor(events=("['created']",))

    # Title
    layout.title.set_text("Cone")

    # Toolbar
    with layout.toolbar as toolbar:
        toolbar.dense = True
        vuetify.VSpacer()
        vuetify.VBtn("Kill server", click=kill_server)
        vuetify.VBtn("Server exception", click=throw_exception)
        vuetify.VBtn("JS Error", click="console.error('test')")

# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import TrameApp  # Base class for a trame app
from trame.ui.vuetify3 import SinglePageLayout  # UI layout
from trame.widgets import vuetify3 as v3, vtk  # UI widgets
from trame.decorators import change  # Method decorators for TrameApp class


class Cone(TrameApp):
    def __init__(self, server=None):
        super().__init__(server)  # enable self.server, self.state, self.ctrl
        self._build_ui()

    def reset_resolution(self):
        self.state.resolution = 6

    @change("resolution")  # When resolution change, execute fn
    def _on_resolution_change(self, resolution, **_):
        print(f"Slider updating resolution to {resolution}")

    def _build_ui(self):
        """Build the user interface"""
        with SinglePageLayout(self.server) as self.ui:
            # Toolbar customization (add-on)
            with self.ui.toolbar as toolbar:
                toolbar.density = "compact"  # Update toolbar attribute
                v3.VSpacer()  # Push things to the right
                v3.VSlider(  # Add slider
                    v_model=(
                        "resolution",
                        6,
                    ),  # bind variable with an initial value of 6
                    min=3,
                    max=60,
                    step=1,  # slider min/max/step
                    density="compact",
                    hide_details=True,  # presentation options
                )

                # Bind methods (from ctrl or self) to 2 buttons with icons
                v3.VBtn(icon="mdi-crop-free", click=self.ctrl.reset_camera)
                v3.VBtn(icon="mdi-undo", click=self.reset_resolution)

            # Content setup
            with self.ui.content:
                # vtk.js view for local rendering
                with vtk.VtkView() as vtk_view:
                    # Bind method to controller
                    self.ctrl.reset_camera = vtk_view.reset_camera
                    with (
                        vtk.VtkGeometryRepresentation()
                    ):  # Add representation to vtk.js view
                        vtk.VtkAlgorithm(  # Add ConeSource to representation
                            vtk_class="vtkConeSource",  # Set attribute value with no JS eval
                            state=(
                                "{ resolution }",
                            ),  # Set attribute value with JS eval
                        )


def main():
    app = Cone()
    app.server.start()


if __name__ == "__main__":
    main()



r"""
Installation requirements:
    pip install trame trame-vuetify
"""

from trame.app import get_server
from trame.app.file_upload import ClientFile
from trame.ui.vuetify import VAppLayout
from trame.widgets import vuetify

server = get_server(client_type="vue2", log_network="./upload-exchange.txt")
state = server.state

with VAppLayout(server):
    vuetify.VFileInput(
        v_model=("file_exchange", None),
    )


@state.change("file_exchange")
def file_uploaded(file_exchange, **kwargs):
    if file_exchange is None:
        return

    file = ClientFile(file_exchange)
    file_name = file_exchange.get("name")
    file_size = file_exchange.get("size")
    file_time = file_exchange.get("lastModified")
    file_mime_type = file_exchange.get("type")
    file_binary_content = file_exchange.get(
        "content"
    )  # can be either list(bytes, ...), or bytes
    print(f"{file_name=}, {file_size=} {file_time=} {file_mime_type=}")
    print(f"file_binary_content=size({len(file_binary_content)})")
    print(file.info)
    print(type(file.content))  # will always be bytes
    # print(file_binary_content)


if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify
"""

import asyncio

from trame.app import asynchronous, get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify

coundown_init = 10

server = get_server(client_type="vue2")
state = server.state
state.trame__title = "Coundown"


@asynchronous.task
async def start_countdown():
    try:
        state.countdown = int(state.countdown)
    except ValueError:
        state.countdown = coundown_init

    while state.countdown > 0:
        with state:
            await asyncio.sleep(0.5)
            state.countdown -= 1


with SinglePageLayout(server) as layout:
    layout.title.set_text("Countdown")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VBtn(
            "Start countdown",
            click=start_countdown,
        )

    with layout.content:
        vuetify.VTextField(
            v_model=("countdown", coundown_init),
            classes="ma-8",
        )

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import trame, vtk, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


def reset_resolution():
    state.resolution = 6


# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

layout = SinglePageLayout(server)

with layout:
    # Validate client life cycle
    trame.LifeCycleMonitor(events=("['created']",))

    layout.icon.click = ctrl.reset_camera
    layout.title.set_text("Cone")
    layout.toolbar.dense = True

    # Toolbar
    with layout.toolbar as toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            hide_details=True,
            v_model=("resolution", 6),
            max=60,
            min=3,
            step=1,
            style="max-width: 300px;",
        )
        vuetify.VSwitch(
            hide_details=True,
            v_model=("$vuetify.theme.dark",),
        )
        with vuetify.VBtn(icon=True, click=reset_resolution):
            vuetify.VIcon("mdi-undo")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            with vtk.VtkView() as view:
                ctrl.reset_camera = view.reset_camera
                with vtk.VtkGeometryRepresentation():
                    vtk.VtkAlgorithm(
                        vtk_class="vtkConeSource", state=("{ resolution }",)
                    )


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



#!/usr/bin/env -S uv run --script
#
# /// script
# requires-python = ">=3.11"
# dependencies = [
#     "trame>=3.10",
#     "trame-components>=2.5",
#     "trame-vtklocal",
#     "trame-vuetify",
#     "vtk==9.5.0rc2",
# ]
#
# [[tool.uv.index]]
# url = "https://wheels.vtk.org"
# ///
import vtk

from trame.app import TrameApp
from trame.ui.vuetify3 import SinglePageWithDrawerLayout
from trame.widgets import vtklocal, trame as tw, vuetify3 as v3
from trame.decorators import change
from trame.assets.remote import HttpFile
from trame.assets.local import to_url

# -----------------------------------------------------------------------------
# Fetch data / files
# -----------------------------------------------------------------------------
BIKE = HttpFile(
    "bike.vtp",
    "https://github.com/Kitware/trame-app-bike/raw/master/data/bike.vtp",
)
TUNNEL = HttpFile(
    "tunnel.vtu",
    "https://github.com/Kitware/trame-app-bike/raw/master/data/tunnel.vtu",
)
IMAGE = HttpFile(
    "seeds.jpg",
    "https://github.com/Kitware/trame-app-bike/raw/master/data/seeds.jpg",
)

if not BIKE.local:
    BIKE.fetch()

if not TUNNEL.local:
    TUNNEL.fetch()

if not IMAGE.local:
    IMAGE.fetch()

# -----------------------------------------------------------------------------
# Constants setup
# -----------------------------------------------------------------------------
P1 = [-0.4, 0, 0.05]
P2 = [-0.4, 0, 1.5]

INITIAL_STATE = {
    "line_widget": {
        "p1": P1,
        "p2": P2,
    },
    "trame__title": "Bike CFD",
    "trame__favicon": to_url(IMAGE.path),
}


# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------
def create_vtk_pipeline():
    K_RANGE = [0.0, 15.6]
    resolution = 50

    renderer = vtk.vtkRenderer()
    renderWindow = vtk.vtkRenderWindow()
    renderWindow.AddRenderer(renderer)
    renderWindow.OffScreenRenderingOn()

    renderWindowInteractor = vtk.vtkRenderWindowInteractor()
    renderWindowInteractor.SetRenderWindow(renderWindow)
    renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

    bikeReader = vtk.vtkXMLPolyDataReader()
    bikeReader.SetFileName(BIKE.path)

    tunnelReader = vtk.vtkXMLUnstructuredGridReader()
    tunnelReader.SetFileName(TUNNEL.path)
    tunnelReader.Update()

    lineSeed = vtk.vtkLineSource()
    lineSeed.SetPoint1(*P1)
    lineSeed.SetPoint2(*P2)
    lineSeed.SetResolution(resolution)
    lineSeed.Update()

    lineWidget = vtk.vtkLineWidget2()
    lineWidgetRep = lineWidget.GetRepresentation()
    lineWidgetRep.SetPoint1WorldPosition(P1)
    lineWidgetRep.SetPoint2WorldPosition(P2)
    lineWidget.SetInteractor(renderWindowInteractor)

    streamTracer = vtk.vtkStreamTracer()
    streamTracer.SetInputConnection(tunnelReader.GetOutputPort())
    streamTracer.SetSourceConnection(lineSeed.GetOutputPort())
    streamTracer.SetIntegrationDirectionToForward()
    streamTracer.SetIntegratorTypeToRungeKutta45()
    streamTracer.SetMaximumPropagation(3)
    streamTracer.SetIntegrationStepUnit(2)
    streamTracer.SetInitialIntegrationStep(0.2)
    streamTracer.SetMinimumIntegrationStep(0.01)
    streamTracer.SetMaximumIntegrationStep(0.5)
    streamTracer.SetMaximumError(0.000001)
    streamTracer.SetMaximumNumberOfSteps(2000)
    streamTracer.SetTerminalSpeed(0.00000000001)

    tubeFilter = vtk.vtkTubeFilter()
    tubeFilter.SetInputConnection(streamTracer.GetOutputPort())
    tubeFilter.SetRadius(0.01)
    tubeFilter.SetNumberOfSides(6)
    tubeFilter.CappingOn()
    tubeFilter.Update()

    bike_mapper = vtk.vtkPolyDataMapper()
    bike_actor = vtk.vtkActor()
    bike_mapper.SetInputConnection(bikeReader.GetOutputPort())
    bike_actor.SetMapper(bike_mapper)
    renderer.AddActor(bike_actor)

    stream_mapper = vtk.vtkPolyDataMapper()
    stream_actor = vtk.vtkActor()
    stream_mapper.SetInputConnection(tubeFilter.GetOutputPort())
    stream_actor.SetMapper(stream_mapper)
    renderer.AddActor(stream_actor)

    lut = vtk.vtkLookupTable()
    lut.SetHueRange(0.7, 0)
    lut.SetSaturationRange(1.0, 0)
    lut.SetValueRange(0.5, 1.0)

    stream_mapper.SetLookupTable(lut)
    stream_mapper.SetColorModeToMapScalars()
    stream_mapper.SetScalarModeToUsePointData()
    stream_mapper.SetArrayName("k")
    stream_mapper.SetScalarRange(K_RANGE)

    renderWindow.Render()
    renderer.ResetCamera()
    renderer.SetBackground(0.4, 0.4, 0.4)

    lineWidget.On()

    return renderWindow, lineSeed, lineWidget, bike_actor


# -----------------------------------------------------------------------------
# Trame app
# -----------------------------------------------------------------------------
class App(TrameApp):
    def __init__(self, server=None):
        super().__init__(server)

        # VTK setup
        self.rw, self.seed, self.widget, self.bike_actor = create_vtk_pipeline()

        # GUI setup
        self._build_ui()

        # Initial state
        self.state.update(INITIAL_STATE)

    @change("bike_opacity")
    def _on_opacity(self, bike_opacity, **_):
        self.bike_actor.property.opacity = bike_opacity
        self.ctrl.view_update()

    @change("line_widget")
    def _on_widget_update(self, line_widget, **_):
        if line_widget is None:
            return

        p1 = line_widget.get("p1")
        p2 = line_widget.get("p2")

        self.seed.SetPoint1(p1)
        self.seed.SetPoint2(p2)

        if line_widget.get("widget_update", False):
            self.widget.representation.point1_world_position = p1
            self.widget.representation.point2_world_position = p2

        self.ctrl.view_update()

    def _build_ui(self):
        with SinglePageWithDrawerLayout(self.server, full_height=True) as layout:
            self.ui = layout  # for jupyter integration

            # Toolbar
            with layout.toolbar as toolbar:
                toolbar.density = "compact"
                layout.title.set_text("Bike CFD")
                v3.VSpacer()
                v3.VSlider(
                    v_model=("bike_opacity", 1),
                    min=0,
                    max=1,
                    step=0.05,
                    density="compact",
                    hide_details=True,
                )
                v3.VBtn(icon="mdi-crop-free", click=self.ctrl.view_reset_camera)

            # Drawer
            with layout.drawer:
                tw.LineSeed(
                    image=to_url(IMAGE.path),
                    point_1=("line_widget.p1",),
                    point_2=("line_widget.p2",),
                    bounds=("[-0.399, 1.80, -1.12, 1.11, -0.43, 1.79]",),
                    update_seed="line_widget = { ...$event, widget_update: 1 }",
                    n_sliders=2,
                )

            # Content
            with layout.content:
                with vtklocal.LocalView(self.rw, throttle_rate=20) as view:
                    self.ctrl.view_update = view.update_throttle
                    self.ctrl.view_reset_camera = view.reset_camera

                    # Bind state to 3D widget interaction event
                    widget_id = view.register_vtk_object(self.widget)
                    view.listeners = (
                        "wasm_listeners",
                        {
                            widget_id: {
                                "InteractionEvent": {
                                    "line_widget": {
                                        "p1": (
                                            widget_id,
                                            "WidgetRepresentation",
                                            "Point1WorldPosition",
                                        ),
                                        "p2": (
                                            widget_id,
                                            "WidgetRepresentation",
                                            "Point2WorldPosition",
                                        ),
                                    }
                                },
                            },
                        },
                    )


def main():
    app = App()
    app.server.start()


if __name__ == "__main__":
    main()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkContourGeneratorFromZarr import vtkContourGeneratorFromZarr
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, vuetify
from trame.widgets import vtk as vtk_widgets

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# set up class to process zarr
# -----------------------------------------------------------------------------

DEFAULT_LEVEL = 3

server.cli.add_argument("--data", help="directory to zarr files", dest="data")
args = server.cli.parse_args()
skin_generator = vtkContourGeneratorFromZarr(args.data)
skin_generator.contourForLevel(DEFAULT_LEVEL)

MAX_LEVEL = skin_generator.getMaxLevel()

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

mapper = vtkPolyDataMapper()
mapper.SetInputConnection(skin_generator.getContour().GetOutputPort())
actor = vtkActor()
actor.SetMapper(mapper)

renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------

state.points = 0
state.cells = 0
state.level = DEFAULT_LEVEL


@state.change("level")
def update_skin(level=DEFAULT_LEVEL, **kwargs):
    nb_points, nb_cells = skin_generator.contourForLevel(level)
    state.points = nb_points
    state.cells = nb_cells
    ctrl.view_update()


def update_reset_level():
    state.level = DEFAULT_LEVEL


def increase_level():
    if state.level < MAX_LEVEL:
        state.level += 1


def decrease_level():
    if 1 < state.level:
        state.level -= 1


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

state.trame__title = "Zarr Skin Generator Demo"


with SinglePageLayout(server) as layout:
    layout.icon.click = "$refs.view.resetCamera()"
    layout.title.set_text("Skin Generator")
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VBtn(
            vuetify.VIcon("mdi-minus"),
            x_small=True,
            icon=True,
            outlined=True,
            click=decrease_level,
            classes="mx-2",
        )
        vuetify.VBtn(
            vuetify.VIcon("mdi-plus"),
            x_small=True,
            icon=True,
            outlined=True,
            click=increase_level,
            classes="mx-2",
        )
        html.Div(
            "Level({{ level }}) "
            "- Points({{parseInt( points ).toLocaleString()}}) "
            "- Cells({{parseInt( cells ).toLocaleString()}})",
            style="min-width: 350px; text-align: right;",
        )
        vuetify.VDivider(vertical=True, classes="mx-2")
        vuetify.VBtn(
            icon=True,
            click=update_reset_level,
            children=[vuetify.VIcon("mdi-undo-variant")],
        )

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            html_view = vtk_widgets.VtkRemoteView(renderWindow, ref="view")
            ctrl.view_update = html_view.update
            ctrl.view_reset_camera = html_view.reset_camera

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



# Ensure this is imported so that mimetypes.init() is decorated
import trame.assets.mimetypes  # noqa: F401
from trame.app.core import get_client, get_server
from trame.app.klass import TrameApp, TrameComponent

__all__ = [
    "get_server",
    "get_client",
    "TrameApp",
    "TrameComponent",
]



#!/usr/bin/env python3

import json
import os
from pathlib import Path


def run(input_path, apps_path, out_path):
    default_command_flags = [
        "--host",
        "${host}",
        "--port",
        "${port}",
        "--authKey",
        "${secret}",
        "--server",
    ]
    default_ready_line = "App running at"

    with open(input_path, "r") as rf:
        input_dict = json.load(rf)

    if not Path(apps_path).exists():
        raise Exception(f"{apps_path} does not exist")

    with open(apps_path, "r") as rf:
        apps_dict = json.load(rf)

    if not apps_dict:
        raise Exception(f"{apps_path} is empty")

    for app_name, config in apps_dict.items():
        if "app" not in config and "cmd" not in config:
            msg = (
                f'In {apps_path}, every app must contain an "app" key '
                f'if a "cmd" is not provided, but "{app_name}" does not'
            )
            raise Exception(msg)

        if "cmd" in config and "args" in config:
            msg = (
                f'In {apps_path}, "args" and "cmd" cannot both be specified. '
                '"args" is for appending extra args to the default "cmd", but '
                '"cmd" is for overriding the command entirely. Error occurred '
                f'in "{app_name}".'
            )
            raise Exception(msg)

        default_cmd = [
            config.get("app", "your_trame_app_package_name")
        ] + default_command_flags
        cmd = config.get("cmd", default_cmd)
        cmd += config.get("args", [])
        ready_line = config.get("ready_line", default_ready_line)

        input_dict.setdefault("apps", {})
        input_dict["apps"][app_name] = {
            "cmd": cmd,
            "ready_line": ready_line,
        }

    if "trame" not in input_dict["apps"]:
        # Make a copy of the first app and put it in PyWebVue, so that
        # the default localhost:9000 web page will use that app.
        first_key = next(iter(input_dict["apps"]))
        input_dict["apps"]["trame"] = input_dict["apps"][first_key]

    # Allow user to override launcher timeout via
    # the TRAME_LAUNCHER_TIMEOUT environment variable
    if "TRAME_LAUNCHER_TIMEOUT" in os.environ:
        timeout = int(os.environ.get("TRAME_LAUNCHER_TIMEOUT", 25))
        input_dict["configuration"]["timeout"] = timeout

    with open(out_path, "w") as wf:
        json.dump(input_dict, wf, indent=2)


if __name__ == "__main__":
    default_input_path = "/opt/trame/default-launcher.json"
    override_input_path = "/deploy/setup/launcher.json"
    apps_path = "/opt/trame/apps.json"
    out_path = "/deploy/server/launcher.json"

    if Path(override_input_path).exists():
        input_path = override_input_path
    else:
        input_path = default_input_path

    run(input_path, apps_path, out_path)



import logging
import logging.handlers
import os
from pathlib import Path

# Required for rendering initialization, not necessary for
# local rendering, but doesn't hurt to include it
import vtkmodules.vtkRenderingOpenGL2  # noqa

# VTK
from vtkmodules.vtkCommonDataModel import vtkDataObject, vtkPolyData
from vtkmodules.vtkFiltersGeometry import vtkGeometryFilter

# Required for interactor initialization
from vtkmodules.vtkInteractionStyle import (
    vtkInteractorStyleSwitch,  # noqa
    vtkInteractorStyleTrackballCamera,
)
from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridReader
from vtkmodules.vtkRenderingAnnotation import vtkCubeAxesActor, vtkScalarBarActor
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkDataSetMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

# Trame
from trame.app import get_server
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import trame, vtk, vuetify

# -----------------------------------------------------------------------------
# Globals
# -----------------------------------------------------------------------------


class Representation:
    Surface = 2
    SurfaceWithEdges = 3


class LookupTable:
    Rainbow = 0


def read_source(path: str, supported_arrays=None):
    if not os.path.isfile(path):
        raise Exception(f"File not found at '{path}'")
    logging.debug(f"Reading source from '{path}'")
    vtk_source = vtkXMLUnstructuredGridReader()
    vtk_source.SetFileName(path)
    vtk_source.Update()

    # Extract Array/Field information from source
    logging.debug("Extracting fields information from source")
    dataset_arrays = []
    fields = [
        (vtk_source.GetOutput().GetPointData(), vtkDataObject.FIELD_ASSOCIATION_POINTS),
        (vtk_source.GetOutput().GetCellData(), vtkDataObject.FIELD_ASSOCIATION_CELLS),
    ]
    for field in fields:
        field_arrays, association = field
        for i in range(field_arrays.GetNumberOfArrays()):
            array = field_arrays.GetArray(i)
            if supported_arrays:
                if array.GetName() in supported_arrays:
                    logging.debug(f"Appending field '{array.GetName()}'")
                    array_range = array.GetRange()
                    _text = array.GetName()
                    dataset_arrays.append(
                        {
                            "text": _text,
                            "variable_name": array.GetName(),
                            "value": i,
                            "range": list(array_range),
                            "type": association,
                        }
                    )
            else:
                _text = array.GetName()
                array_range = array.GetRange()
                dataset_arrays.append(
                    {
                        "text": _text,
                        "variable_name": array.GetName(),
                        "value": i,
                        "range": list(array_range),
                        "type": association,
                    }
                )
    if not dataset_arrays:
        raise Exception("Error reading data")
    logging.debug("Extracted fields: [")
    for arr in dataset_arrays:
        logging.debug(
            f"\t[index: {arr['value']}, variable: {arr['variable_name']}, range: {arr['range']}, type: {arr['text']}],"
        )
    logging.debug("]")

    # Sort ascending by field 'variable_name'
    logging.info("Sorting arrays by alphabetic ascending")
    dataset_arrays = sorted(
        dataset_arrays, key=lambda d: d["variable_name"]
    )  # ascending
    # Update index after sorting
    logging.info("Updating index after sorting")
    for index, arr in enumerate(dataset_arrays):
        dataset_arrays[index]["value"] = int(index)
    logging.debug("Sorted fields: [")
    for arr in dataset_arrays:
        logging.debug(
            f"\t[index: {arr['value']}, variable: {arr['variable_name']}, range: {arr['range']}, type: {arr['text']}],"
        )
    logging.debug("]")

    # Append solid color
    dataset_arrays.append(
        {
            "text": "Solid Color",
            "variable_name": "solid_color",
            "value": len(dataset_arrays),
            "range": [0, 1],
            "type": vtkDataObject.FIELD_ASSOCIATION_POINTS,
        }
    )
    return vtk_source, dataset_arrays


def getBounds(reader_port):
    # Get data center and bounds
    geometryFilter = vtkGeometryFilter()
    geometryFilter.SetInputConnection(reader_port)
    geometryFilter.Update()
    polyData = geometryFilter.GetOutput()
    polyDataBounds = [0, 0, 0, 0, 0, 0]  # [xmin, xmax, ymin, ymax, zmin, zmax]
    polyData.GetCellsBounds(polyDataBounds)  # modifies the 'polyDataBounds' variable
    return polyData, polyDataBounds


def set_color_preset(lut, value):
    # Set HSV values
    if value == LookupTable.Rainbow:
        lut.SetHueRange(0.666, 0.0)  # H - hue
        lut.SetSaturationRange(1.0, 1.0)  # S - saturation
        lut.SetValueRange(1.0, 1.0)  # V - value
    lut.Build()
    return lut


def get_default_array(arrays, reader_port):
    existing_field_names = []
    for i in range(len(arrays)):
        existing_field_names.append(arrays[i].get("variable_name"))
    default_array_idx = 0
    default_array = arrays[default_array_idx]
    del existing_field_names
    logging.debug("Getting bounds")
    abs_min, abs_max = default_array.get("range")
    polyData, polyDataBounds = getBounds(reader_port)
    return default_array, default_array_idx, abs_min, abs_max, polyData, polyDataBounds


# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------
logging.debug("Setting up Trame")
server = get_server(client_type="vue2")
state, ctrl, ui = server.state, server.controller, server.ui
state.setdefault("active_ui", None)  # set the default active UI

# -----------------------------------------------------------------------------
# State management
# -----------------------------------------------------------------------------
TITLE = "Trame test"
state.trame__title = TITLE
# Set defaults
state.update(
    {
        # Common parameters
        "color_preset": LookupTable.Rainbow,
        "dataset_arrays": [],
        "default_color_array_idx": None,
        "color_array_idx": None,
        # Mesh
        "mesh_opacity": 1.0,
    }
)

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------
# ***** Create renderer *****
logging.debug("Creating renderer")
renderer = vtkRenderer()
renderer.GradientBackgroundOn()
render_window = vtkRenderWindow()
render_window.AddRenderer(renderer)
render_window_interactor = vtkRenderWindowInteractor()
interactor_style = vtkInteractorStyleTrackballCamera()
render_window_interactor.SetInteractorStyle(interactor_style)
render_window_interactor.SetRenderWindow(render_window)

# Initialize actors and mappers
dataset_mapper = vtkDataSetMapper()
mesh_actor = vtkActor()
scalarBarActor = vtkScalarBarActor()
axes_actor = vtkCubeAxesActor()
polyData = vtkPolyData()
polyDataBounds = [0, 0, 0, 0, 0, 0]  # [xmin, xmax, ymin, ymax, zmin, zmax]

# ***** Read source, extract fields and compute some values (min, max and bounds) *****
input_file_path = str(Path(__file__).with_name("patch_antenna.vtu").resolve())
vtk_source, dataset_arrays = read_source(path=input_file_path)
(
    default_array,
    default_array_idx,
    abs_min,
    abs_max,
    polyData,
    polyDataBounds,
) = get_default_array(
    dataset_arrays,
    vtk_source.GetOutputPort(),
)
# Update state
state.dataset_arrays = dataset_arrays
logging.debug("Setting the default array index")
state.default_color_array_idx = default_array_idx
state.color_array_idx = default_array_idx
logging.debug("Setting some values of the default array to state")
# Initialize absolute min and max values
state.abs_min = abs_min
state.abs_max = abs_max
logging.debug(
    f"New values to state: [abs_min,abs_max]=[{state.abs_min}, {state.abs_max}]"
)
state.bounds = polyDataBounds
logging.debug(f"New values to state: bounds={state.bounds}")

# ***** Dataset, color map and scalar bar *****
# ***** Dataset *****
logging.debug("Configuring dataset mapper")
dataset_mapper.SetInputConnection(vtk_source.GetOutputPort())
dataset_mapper.ScalarVisibilityOn()
scalar_range = vtk_source.GetOutput().GetScalarRange()
dataset_mapper.SetScalarRange(scalar_range)
# Color map
logging.debug("Configuring color map")
dataset_lut = set_color_preset(dataset_mapper.GetLookupTable(), LookupTable.Rainbow)
dataset_mapper.SetLookupTable(dataset_lut)

# ***** Mesh *****
logging.debug("Configuring mesh actor")
mesh_actor.SetMapper(dataset_mapper)
# Mesh: Setup default representation to surface
mesh_actor.GetProperty().SetRepresentationToSurface()
mesh_actor.GetProperty().SetPointSize(1)
mesh_actor.GetProperty().EdgeVisibilityOn()
renderer.AddActor(mesh_actor)

# *****  Scalar bar *****
logging.debug("Configuring scalar bar")
scalarBarActor.SetLookupTable(dataset_mapper.GetLookupTable())
scalarBarActor.SetNumberOfLabels(7)
scalarBarActor.UnconstrainedFontSizeOn()
scalarBarActor.SetMaximumWidthInPixels(100)
scalarBarActor.SetMaximumHeightInPixels(800 // 3)
scalarBarActor.SetTitle(default_array.get("text"))
renderer.AddActor2D(scalarBarActor)

# ***** Axes - boundaries, camera, and styling *****
logging.debug("Configuring axes")
axes_actor.SetBounds(mesh_actor.GetBounds())
axes_actor.SetCamera(renderer.GetActiveCamera())
axes_actor.SetXLabelFormat("%6.1f")
axes_actor.SetYLabelFormat("%6.1f")
axes_actor.SetZLabelFormat("%6.1f")
axes_actor.SetFlyModeToOuterEdges()
renderer.AddActor(axes_actor)


# -----------------------------------------------------------------------------
# Useful methods
# -----------------------------------------------------------------------------
def reset():
    state.update(
        {
            "color_preset": LookupTable.Rainbow,
            "default_color_array_idx": 0,
            "color_array_idx": 0,
        }
    )
    state.active_ui = "colormap"  # the default active UI
    set_default_visibility()
    ctrl.view_update()
    ctrl.view_reset_camera()


# Visibility
def set_default_visibility():
    scalarBarActor.SetVisibility(True)
    mesh_actor.SetVisibility(True)


def visibility_change(event):
    _id = event["id"]
    _visibility = event["visible"]
    if _id == "1":  # Color Map
        scalarBarActor.SetVisibility(_visibility)
        if _visibility:  # If colormap has been enabled, color by the default array
            update_color_by_name(state.default_color_array_idx)
            state.active_ui = "colormap"
        else:  # If colormap has been disabled, color by a solid color
            update_color_by_name(len(state.dataset_arrays) - 1)
            state.active_ui = "nothing"
            # Disable scalar bar
            scalarBarActor.SetVisibility(False)
    elif _id == "2":  # Mesh
        mesh_actor.SetVisibility(_visibility)
        state.active_ui = "mesh" if _visibility else "nothing"

    ctrl.view_update()


# Color By Callbacks
def update_colormap(actor, arr):
    logging.debug("INIT update_colormap")
    logging.debug(f"Setting colormap for field '{arr.get('text')}'")
    _min, _max = arr.get("range")
    # Update dataset mapper
    mapper = actor.GetMapper()
    if not mapper:
        return
    mapper.SelectColorArray(arr.get("variable_name"))
    print("SetScalarRange", _min, _max)
    mapper.SetScalarRange([_min, _max])
    if arr.get("type") == vtkDataObject.FIELD_ASSOCIATION_POINTS:
        mapper.SetScalarModeToUsePointFieldData()
    else:
        mapper.SetScalarModeToUseCellFieldData()
    # Update mesh actor
    logging.debug("END update_colormap")


# Selection
def actives_change(ids):
    _id = ids[0]
    if _id == "1" and scalarBarActor.GetVisibility():  # Color Map
        state.active_ui = "colormap"
    elif _id == "2" and mesh_actor.GetVisibility():  # Mesh
        state.active_ui = "mesh"
    else:
        state.active_ui = "nothing"


# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------
# ** INIT Common callbacks **
@state.change("color_array_idx")
def update_color_by_name(color_array_idx, **kwargs):
    logging.debug("INIT update_color_by_name")
    if (
        mesh_actor.GetMapper()
    ):  # only if some geometry was loaded (after initializing the GUI)
        # Get the array (field) for the given index
        logging.debug(f"Getting array with index '{color_array_idx}'")
        _array = state.dataset_arrays[color_array_idx]
        # Disable/Enable scalar bar according to the selected field: disable scalar bar is solid color was chosen
        if color_array_idx == (len(state.dataset_arrays) - 1):  # disable scalar bar
            scalarBarActor.SetVisibility(False)
        else:  # enable scalar bar
            scalarBarActor.SetVisibility(True)
        # Update scalar bar title
        logging.debug(f"Updating scalar bar title to '{_array.get('text')}'")
        scalarBarActor.SetTitle(_array.get("text"))
        # Update color map for mesh
        logging.debug("Updating colormap for mesh actor")
        update_colormap(mesh_actor, _array)
        # Update scalar bar actor
        scalarBarActor.SetLookupTable(mesh_actor.GetMapper().GetLookupTable())
        # Update state
        state.color_array_idx = color_array_idx
        # Initialize custom min and max to absolute min and max
        logging.debug("Updating bounds")
        # Initialize absolute min and max values
        _min, _max = _array.get("range")
        state.abs_min = _min
        state.abs_max = _max
        logging.debug(
            f"New values to state: [abs_min,abs_max]=[{state.abs_min}, {state.abs_max}]"
        )

        # Update view
        logging.debug("Updating view")
        ctrl.view_update()

    logging.debug("END update_color_by_name")


@state.change("color_preset")
def update_color_preset(color_preset, **kwargs):
    logging.debug("INIT update_color_preset")
    logging.debug(f"Updating color preset to '{color_preset}'")
    if mesh_actor.GetMapper():
        mesh_actor.GetMapper().SetLookupTable(
            set_color_preset(mesh_actor.GetMapper().GetLookupTable(), color_preset)
        )
    ctrl.view_update()
    logging.debug("END update_color_preset")


# ** END Common callbacks **


# ** INIT Axes callbacks **
@state.change("axes_visibility")
def update_axes_visibility(axes_visibility: bool, **kwargs):
    logging.debug("INIT update_axes_visibility")
    logging.debug(f"Updating axes visibility to '{axes_visibility}'")
    axes_actor.SetVisibility(axes_visibility)
    ctrl.view_update()
    logging.debug("END update_axes_visibility")


# ** END Axes callbacks **


# ** INIT Mesh callbacks **
@state.change("mesh_representation")
def update_mesh_representation(mesh_representation, **kwargs):
    logging.debug(f"Updating mesh representation to '{mesh_representation}'")
    update_representation(mesh_actor, mesh_representation)
    ctrl.view_update()


def update_representation(actor, mode):
    logging.debug("INIT update_representation")
    _property = actor.GetProperty()
    if mode == Representation.Surface:
        _property.SetRepresentationToSurface()
        _property.SetPointSize(1)
        _property.EdgeVisibilityOff()
    elif mode == Representation.SurfaceWithEdges:
        _property.SetRepresentationToSurface()
        _property.SetPointSize(1)
        _property.EdgeVisibilityOn()
    logging.debug("END update_representation")


# Opacity Callbacks
@state.change("mesh_opacity")
def update_mesh_opacity(mesh_opacity, **kwargs):
    logging.debug(f"Updating mesh opacity to '{mesh_opacity}'")
    mesh_actor.GetProperty().SetOpacity(mesh_opacity)
    ctrl.view_update()


# ** END Mesh callbacks **


# -----------------------------------------------------------------------------
# GUI ELEMENTS
# -----------------------------------------------------------------------------
def standard_buttons():
    vuetify.VCheckbox(
        v_model=("axes_visibility", True),
        on_icon="mdi-cube-outline",
        off_icon="mdi-cube-off-outline",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    with vuetify.VBtn(icon=True, click="$refs.view.resetCamera()"):
        vuetify.VIcon("mdi-crop-free")


def ui_card(title, ui_name):
    with vuetify.VCard(v_show=f"active_ui == '{ui_name}'"):
        vuetify.VCardTitle(
            title,
            classes="grey lighten-1 py-1 grey--text text--darken-3",
            style="user-select: none; cursor: pointer",
            hide_details=True,
            dense=True,
        )
        content = vuetify.VCardText(classes="py-2")
    return content


def colormap_card():
    with ui_card(title="Color Map", ui_name="colormap"):
        # Field and colormap
        with vuetify.VRow(classes="pt-2", dense=True):
            with vuetify.VCol(cols="12"):
                vuetify.VSelect(
                    # Color By
                    label="Field",
                    v_model=("color_array_idx", state.default_color_array_idx),
                    items=("array_list", state.dataset_arrays),
                    hide_details=True,
                    dense=True,
                    outlined=True,
                    classes="pt-1",
                )


def mesh_card():
    with ui_card(title="Geometry", ui_name="mesh"):
        # Representation
        with vuetify.VRow(classes="pt-2", dense=True):
            vuetify.VSelect(
                v_model=("mesh_representation", Representation.Surface),
                items=(
                    "representations",
                    [{"text": "Surface", "value": 2}, {"text": "Mesh", "value": 3}],
                ),
                label="Representation",
                hide_details=True,
                dense=True,
                outlined=True,
                classes="pt-1",
            )
        # Opacity
        vuetify.VSlider(
            v_model=("mesh_opacity", state.mesh_opacity),
            min=0,
            max=1,
            step=0.01,
            label="Opacity",
            classes="mt-1",
            hide_details=True,
            dense=True,
        )


def left_menu():
    trame.GitTree(
        sources=(
            "pipeline",
            [
                {"id": "1", "parent": "0", "visible": 1, "name": "Color Map"},
                {"id": "2", "parent": "1", "visible": 1, "name": "Geometry"},
            ],
        ),
        actives_change=(actives_change, "[$event]"),
        visibility_change=(visibility_change, "[$event]"),
    )
    vuetify.VDivider(classes="mb-2")
    colormap_card()
    mesh_card()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------
set_default_visibility()
renderer.ResetCamera()

# Layout content
logging.debug("Configuring GUI")
with SinglePageWithDrawerLayout(server) as layout:
    # -----------------------------------------------------------------------------
    # Toolbar
    # -----------------------------------------------------------------------------
    # Toolbar title
    layout.title.set_text("Trame test")
    layout.icon.click = reset  # call reset() function

    # Toolbar right-side buttons
    with layout.toolbar as toolbar:
        vuetify.VSpacer()
        vuetify.VDivider(vertical=True, classes="mx-2")
        standard_buttons()
        # Loading bar
        vuetify.VProgressLinear(
            indeterminate=True,
            absolute=True,
            bottom=True,
            active=("trame__busy",),
        )

    # Drawer components
    with layout.drawer as drawer:
        drawer.width = 325
        left_menu()

    # Content
    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            # view = vtk.VtkLocalView(render_window, interactive_ratio=1)
            view = vtk.VtkRemoteView(render_window, interactive_ratio=1)
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera
            ctrl.on_server_ready.add(view.update)

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import pyvista as pv
from pyvista.trame.ui import plotter_ui
from vtkmodules.vtkFiltersSources import vtkSphereSource
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
)

from trame.app import get_server
from trame.widgets import vtk as vtk_widgets

CLIENT_TYPE = "vue3"
USE_PV = False
RADIUS = 0.1

if CLIENT_TYPE == "vue2":
    from trame.ui.vuetify2 import SinglePageLayout
    from trame.widgets import vuetify2 as vuetify
else:
    from trame.ui.vuetify3 import SinglePageLayout
    from trame.widgets import vuetify3 as vuetify


class App:
    def __init__(self, server=None):
        self.server = get_server(server, client_type=CLIENT_TYPE)

        if USE_PV:
            self.plotter = pv.Plotter()
        else:
            self.renderWindow = vtkRenderWindow()
            self.renderer = vtkRenderer()
            self.renderWindow.AddRenderer(self.renderer)
        self.sources_counter = 0
        self.ui()

    @property
    def ctrl(self):
        return self.server.controller

    @property
    def state(self):
        return self.server.state

    def add_actor(self):
        center = (self.sources_counter * RADIUS * 2.1, 0, 0)
        self.sources_counter += 1
        if USE_PV:
            mapper = pv.DataSetMapper(pv.Sphere(radius=RADIUS, center=center))
            actor = pv.Actor(mapper=mapper)
            self.plotter.add_actor(actor)
        else:
            _source = vtkSphereSource()
            _source.SetRadius(RADIUS)
            _source.SetCenter(*center)
            mapper = vtkPolyDataMapper()
            actor = vtkActor()
            mapper.SetInputConnection(_source.GetOutputPort())
            actor.SetMapper(mapper)
            self.renderer.AddActor(actor)

        self.ctrl.view_update()
        self.ctrl.view_reset_camera()

    def ui(self):
        with SinglePageLayout(self.server) as layout:
            self.layout = layout
            layout.icon.click = self.ctrl.view_reset_camera

            with layout.content:
                with vuetify.VCol():
                    vuetify.VBtn("AddActor", click=self.add_actor)
                with vuetify.VContainer(
                    fluid=True,
                    classes="pa-0 fill-height",
                ):
                    view = (
                        plotter_ui(self.plotter, mode="client")
                        if USE_PV
                        else vtk_widgets.VtkLocalView(self.renderWindow)
                    )
                    self.ctrl.view_update = view.update
                    self.ctrl.view_reset_camera = view.reset_camera


if __name__ == "__main__":
    app = App()
    app.add_actor()
    app.add_actor()
    app.server.start()



import vtk
import vtkmodules.vtkRenderingOpenGL2  # noqa (needed for vtkHardwareSelector)
from vtkmodules.vtkFiltersCore import vtkTubeFilter
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Generate dataset
# -----------------------------------------------------------------------------

points = vtk.vtkPoints()
points.SetNumberOfPoints(4)
line = vtk.vtkLine()
lines = vtk.vtkCellArray()

# create first line segment
points.SetPoint(0, 0, 0, 0)
line.GetPointIds().SetId(0, 0)

points.SetPoint(1, 1, 1, 1)
line.GetPointIds().SetId(1, 1)

lines.InsertNextCell(line)

# create second line segment
points.SetPoint(2, 1, 1, 1)
line.GetPointIds().SetId(0, 2)

points.SetPoint(3, 2, 2, 2)
line.GetPointIds().SetId(1, 3)

lines.InsertNextCell(line)

linesPolyData = vtk.vtkPolyData()
linesPolyData.SetPoints(points)
linesPolyData.SetLines(lines)

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Interactions
# -----------------------------------------------------------------------------


def foo(pickData):
    print("hello")


# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

tubes_filter = vtkTubeFilter()
tubes_filter.SetInputData(linesPolyData)
tubes_filter.SetRadius(2)
tubes_filter.SetNumberOfSides(3)
tubes_filter.Update()

mapper = vtkPolyDataMapper()
mapper.SetInputConnection(tubes_filter.GetOutputPort())
actor = vtkActor()
actor.SetMapper(mapper)

renderer = vtkRenderer()
renderer.SetBackground(1, 1, 1)
renderer.AddActor(actor)

render_window = vtkRenderWindow()
render_window.AddRenderer(renderer)

rw_interactor = vtkRenderWindowInteractor()
rw_interactor.SetRenderWindow(render_window)
rw_interactor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

renderer.ResetCamera()

# -----------------------------------------------------------------------------
# GUI layout
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    with layout.content:
        with vuetify.VContainer(fluid=True, classes="fill-height pa-0 ma-0"):
            view = vtk_widgets.VtkRemoteView(
                render_window,
                interactor_events=("events", ["LeftButtonPress"]),
                LeftButtonPress=(foo, "[utils.vtk.event($event)]"),
            )
            view.update
            view.reset_camera

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersSources import vtkConeSource
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

USING_WITH = True
server = get_server(client_type="vue2")
ctrl = server.controller


class App:
    def __init__(self, server):
        self.server = server
        self.layout = None
        self.container = None

        self.rw = vtkRenderWindow()
        self.renderer = vtkRenderer()
        self.rw.AddRenderer(self.renderer)
        self.cone_source = None

        # Just because we are deferring UI initialization
        vtk_widgets.initialize(server)
        vuetify.initialize(server)

        self.ctrl.add("on_server_ready")(self.ui)
        self.state.change("resolution")(self.update_resolution)

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    def update_resolution(self, resolution, **kwargs):
        if self.cone_source:
            self.cone_source.SetResolution(resolution)
            self.ctrl.view_update()

    def add_cone(self):
        self.state.cone_disabled = True
        self.cone_source = vtkConeSource()
        mapper = vtkPolyDataMapper()
        mapper.SetInputConnection(self.cone_source.GetOutputPort())
        actor = vtkActor()
        actor.SetMapper(mapper)
        self.renderer.AddActor(actor)
        self.renderer.ResetCamera()

        if USING_WITH:
            with self.layout:  # Will call self.layout.flush_content() on_exit
                with self.container:
                    with vtk_widgets.VtkLocalView(self.rw) as view:
                        self.ctrl.view_update = view.update
                        self.ctrl.view_reset_camera = view.reset_camera
        else:
            # Could also be
            view = vtk_widgets.VtkLocalView(self.rw)
            self.ctrl.view_update = view.update
            self.ctrl.view_reset_camera = view.reset_camera
            self.container.add_child(view)
            self.layout.flush_content()

    def ui(self, **kwargs):
        with SinglePageLayout(self.server) as layout:
            self.layout = layout
            layout.title.set_text("Hello trame")
            layout.icon.click = self.ctrl.view_reset_camera

            with layout.toolbar:
                vuetify.VSpacer()
                vuetify.VSlider(
                    v_show=("cone_disabled",),
                    v_model=("resolution", 6),
                    max=60,
                    dense=True,
                    hide_details=True,
                )
                vuetify.VBtn(
                    "Cone Pipeline",
                    disabled=("cone_disabled", False),
                    click=self.add_cone,
                )

            with layout.content:
                self.container = vuetify.VContainer(
                    fluid=True,
                    classes="pa-0 fill-height",
                )


if __name__ == "__main__":
    app = App(server)
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/ContourGeometry/DynamicLocalRemoteRendering.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/3852cba56cd63f6efa684d5e5fb00881a6111e81

Installation requirements:
    pip install trame trame-vuetify trame-vtk vtk
"""

from pathlib import Path

import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersCore import vtkContourFilter

# VTK factory initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkIOXML import vtkXMLImageDataReader
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "VTK contour"
ctrl.on_server_ready.add(ctrl.view_update)

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

data_directory = Path(__file__).parent.parent.with_name("data")
head_vti = data_directory / "head.vti"

reader = vtkXMLImageDataReader()
reader.SetFileName(head_vti)
reader.Update()

contour = vtkContourFilter()
contour.SetInputConnection(reader.GetOutputPort())
contour.SetComputeNormals(1)
contour.SetComputeScalars(0)

# Extract data range => Update store/state
data_range = reader.GetOutput().GetPointData().GetScalars().GetRange()
contour_value = 0.5 * (data_range[0] + data_range[1])
state.data_range = data_range
state.contour_value = contour_value

# Configure contour with valid values
contour.SetNumberOfContours(1)
contour.SetValue(0, contour_value)

# Rendering setup
renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

mapper = vtkPolyDataMapper()
actor = vtkActor()
mapper.SetInputConnection(contour.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("contour_value")
def update_contour(contour_value, **kwargs):
    contour.SetValue(0, contour_value)
    ctrl.view_update_image()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

modes = (
    ("auto", "mdi-autorenew"),
    ("local", "mdi-rotate-3d"),
    ("remote", "mdi-image"),
)


with SinglePageLayout(server) as layout:
    layout.title.set_text("Contour")
    layout.icon.click = ctrl.view_reset_camera

    with layout.toolbar:
        vuetify.VSpacer()

        with vuetify.VBtnToggle(
            v_model=("override", "auto"),
            dense=True,
            mandatory=True,
        ):
            for entry in modes:
                with vuetify.VBtn(value=entry[0]):
                    vuetify.VIcon(entry[1])

        vuetify.VSpacer()
        vuetify.VSlider(
            v_model="contour_value",
            min=("data_range[0]",),
            max=("data_range[1]",),
            hide_details=True,
            dense=True,
            style="max-width: 300px",
            start="trigger('demoAnimateStart')",
            end="trigger('demoAnimateStop')",
            change=ctrl.view_update,
        )
        vuetify.VSwitch(
            v_model="$vuetify.theme.dark",
            hide_details=True,
        )

        with vuetify.VBtn(icon=True, click=ctrl.view_reset_camera):
            vuetify.VIcon("mdi-crop-free")

        vuetify.VProgressLinear(
            indeterminate=True,
            absolute=True,
            bottom=True,
            active=("trame__busy",),
        )

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            view = vtk_widgets.VtkRemoteLocalView(
                renderWindow,
                namespace="demo",
                # second arg is to force the view to start in "local" mode
                mode=("override === 'auto' ? demoMode : override", "local"),
            )
            ctrl.view_reset_camera = view.reset_camera
            ctrl.view_update = view.update
            ctrl.view_update_image = view.update_image

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import panel as pn
import pyvista as pv
from pyvista import examples

pn.extension("vtk", design="material", sizing_mode="stretch_width", template="material")

pn.state.template.config.raw_css.append(
    """
#main {
  padding: 0;
}"""
)

m = examples.download_st_helens().warp_by_scalar()

# default camera position
cpos = [
    (567000.9232163235, 5119147.423216323, 6460.423216322832),
    (562835.0, 5114981.5, 2294.5),
    (-0.4082482904638299, -0.40824829046381844, 0.8164965809277649),
]

# pyvista plotter
pl = pv.Plotter(notebook=True)
actor = pl.add_mesh(m, smooth_shading=True, lighting=True)
pl.camera_position = cpos  # set camera position

# save initial camera properties
renderer = list(pl.ren_win.GetRenderers())[0]
initial_camera = renderer.GetActiveCamera()
initial_camera_pos = {
    "focalPoint": initial_camera.GetFocalPoint(),
    "position": initial_camera.GetPosition(),
    "viewUp": initial_camera.GetViewUp(),
}

# Panel creation using the VTK Scene created by the plotter pyvista
orientation_widget = True
enable_keybindings = True
vtkpan = pn.pane.VTK(
    pl.ren_win,
    margin=0,
    sizing_mode="stretch_both",
    orientation_widget=orientation_widget,
    enable_keybindings=enable_keybindings,
    min_height=600,
)
vtkpan

# =============================================================================
# Creation of a mapping between Custom name and the VTK object reference
# =============================================================================

actor_ref = ["None", actor.__this__]
actor_names = ["None", "St Helen"]
actor_opts = {k: v for k, v in zip(actor_names, actor_ref)}

options = {}
actor_selection = pn.widgets.Select(
    value=None, options=actor_opts, margin=0, name="Actor Selection"
)
actor_selection

# =============================================================================
# Scene Layout
# =============================================================================

color = "".join(
    ["#"] + ["{:02x}".format(int(v * 255)) for v in pl.background_color[:3]]
)
bind_and_orient = pn.widgets.CheckBoxGroup(
    value=["Orientation Widget", "Key Bindings"],
    options=["Orientation Widget", "Key Bindings"],
)
reset_camera = pn.widgets.Button(name="Reset Camera")
background_color = pn.widgets.ColorPicker(value=color, name="Background Color")
scene_props = pn.WidgetBox(
    bind_and_orient, reset_camera, background_color, sizing_mode="stretch_width"
)

# Light properties
light_box_title = pn.widgets.StaticText(value="Light properties")
light_type = pn.widgets.Select(
    value="HeadLight", options=["HeadLight", "SceneLight", "CameraLight"]
)
light_intensity = pn.widgets.FloatSlider(start=0, end=1, value=1, name="Intensity")
light_props = pn.WidgetBox(
    light_box_title, light_type, light_intensity, sizing_mode="stretch_width"
)

pn.Column(scene_props, light_props, max_width=320)


# =============================================================================
# layout actor props
# =============================================================================

opacity = pn.widgets.FloatSlider(value=1, start=0, end=1, name="Opacity", disabled=True)
lighting = pn.widgets.Toggle(value=True, name="Lighting", disabled=True)
interpolation = pn.widgets.Select(
    value="Phong", options=["Flat", "Phong"], name="Interpolation", disabled=True
)
edges = pn.widgets.Toggle(value=False, name="Show Edges", disabled=True)
edges_color = pn.widgets.ColorPicker(value="#ffffff", name="Edges Color", disabled=True)
representation = pn.widgets.Select(
    value="Surface",
    options=["Points", "Wireframe", "Surface"],
    name="Representation",
    disabled=True,
)
frontface_culling = pn.widgets.Toggle(
    value=False,
    name="Frontface Culling",
    disabled=True,
)
backface_culling = pn.widgets.Toggle(
    value=False,
    name="Backface Culling",
    disabled=True,
)
ambient = pn.widgets.FloatSlider(
    value=0,
    start=-1,
    end=1,
    name="Ambient",
    disabled=True,
)
diffuse = pn.widgets.FloatSlider(
    value=1,
    start=0,
    end=2,
    name="Diffuse",
    disabled=True,
)
specular = pn.widgets.FloatSlider(
    value=0, start=0, end=10, name="Specular", disabled=True
)
specular_power = pn.widgets.FloatSlider(
    value=100, start=0, end=100, name="Specular Power", disabled=True
)

actor_props = pn.WidgetBox(
    opacity,
    lighting,
    interpolation,
    edges,
    edges_color,
    representation,
    frontface_culling,
    backface_culling,
    ambient,
    diffuse,
    specular,
    specular_power,
)

actor_props


# =============================================================================
# Linking
# =============================================================================

light_type.jslink(
    vtkpan,
    code={
        "value": """
const light = target.renderer_el.getRenderer().getLights()[0]
if (source.value == 'HeadLight')
    light.setLightTypeToHeadLight()
else if (source.value == 'CameraLight')
    light.setLightTypeToCameraLight()
else if (source.value == 'SceneLight')
    light.setLightTypeToSceneLight()
target.renderer_el.getRenderWindow().render()
"""
    },
)

light_intensity.jslink(
    vtkpan,
    code={
        "value": """
const light = target.renderer_el.getRenderer().getLights()[0]
light.setIntensity(source.value)
target.renderer_el.getRenderWindow().render()
"""
    },
)


bind_and_orient.jslink(
    vtkpan,
    code={
        "active": """
target.orientation_widget = source.active.includes(0)
target.enable_keybindings = source.active.includes(1)
"""
    },
)

reset_camera.js_on_click(
    args={"target": vtkpan, "initial_camera": initial_camera_pos},
    code="target.camera = initial_camera",
)

background_color.jslink(
    vtkpan,
    code={
        "value": """
const hextoarr = (color) => {
    return [
        parseInt(color.slice(1,3),16)/255,
        parseInt(color.slice(3,5),16)/255,
        parseInt(color.slice(5,7),16)/255
    ]
}
target.renderer_el.getRenderer().setBackground(hextoarr(source.color))
target.renderer_el.getRenderWindow().render()
"""
    },
)

opacity.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setOpacity(source.value)
    target.renderer_el.getRenderWindow().render()
}
""",
)

lighting.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setLighting(source.active)
    target.renderer_el.getRenderWindow().render()
}
""",
)

edges.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setEdgeVisibility(source.active)
    target.renderer_el.getRenderWindow().render()
}
""",
)

interpolation.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    if(source.value=="Flat"){
        actor.getProperty().setInterpolationToFlat()
    }else{
        actor.getProperty().setInterpolationToPhong()
    }
    target.renderer_el.getRenderWindow().render()
}
""",
)

edges_color.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const hextoarr = (color) => {
        return [
            parseInt(color.slice(1,3),16)/255,
            parseInt(color.slice(3,5),16)/255,
            parseInt(color.slice(5,7),16)/255
        ]
    }
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setEdgeColor(hextoarr(source.color))
    target.renderer_el.getRenderWindow().render()
}
""",
)

representation.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    if(source.value=="Points"){
        actor.getProperty().setRepresentationToPoints()
    }else if(source.value=="Wireframe"){
        actor.getProperty().setRepresentationToWireframe()
    }else if(source.value=="Surface"){
        actor.getProperty().setRepresentationToSurface()
    }
    target.renderer_el.getRenderWindow().render()
}
""",
)

frontface_culling.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setFrontfaceCulling(source.active)
    target.renderer_el.getRenderWindow().render()
}
""",
)

backface_culling.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setBackfaceCulling(source.active)
    target.renderer_el.getRenderWindow().render()
}
""",
)

ambient.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setAmbient(source.value)
    target.renderer_el.getRenderWindow().render()
}
""",
)

diffuse.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setDiffuse(source.value)
    target.renderer_el.getRenderWindow().render()
}
""",
)

specular.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setSpecular(source.value)
    target.renderer_el.getRenderWindow().render()
}
""",
)

specular_power.jscallback(
    args={"target": vtkpan, "actor_selection": actor_selection},
    value="""
if (actor_selection.value!="None"){
    const actor = target.getActors(actor_selection.value)[0]
    actor.getProperty().setSpecularPower(source.value)
    target.renderer_el.getRenderWindow().render()
}
""",
)

actor_selection.jslink(
    target=vtkpan,
    code={
        "value": """
if (source.value!="None"){
    const actor = target.getActors(source.value)[0]
    target.outline.setInputData(actor.getMapper().getInputData())
    target.renderer_el.getRenderer().addActor(target.outline_actor)

    //synchronize actor props and widgets values
    const properties = actor.getProperty()
    opacity.setv({value: properties.getOpacity()}, {silent: true})
    lighting.setv({active: !!properties.getLighting()}, {silent: true})
    edges.active = !!properties.getEdgeVisibility()
    const actor_color = "#"
        + properties.getEdgeColor().map(
            (c) => ("0" + Math.round(255*c).toString(16,2)
        ).slice(-2)).join('')
    edges_color.setv({color: actor_color}, {silent: true})
    const interp_string = properties.getInterpolationAsString()
    interpolation.setv({value: interp_string[0] + interp_string.slice(1).toLocaleLowerCase()}, {silent: true})
    const repr_string = properties.getRepresentationAsString()
    representation.setv({value: repr_string[0] + repr_string.slice(1).toLocaleLowerCase()}, {silent: true})
    frontface_culling.setv({active: !!properties.getFrontfaceCulling()}, {silent: true})
    backface_culling.setv({active: !!properties.getBackfaceCulling()}, {silent: true})
    ambient.setv({value: properties.getAmbient()}, {silent: true})
    diffuse.setv({value: properties.getDiffuse()}, {silent: true})
    specular.setv({value: properties.getSpecular()}, {silent: true})
    specular_power.setv({value: properties.getSpecularPower()}, {silent: true})
    //enable widgets modifications
    opacity.disabled = false
    lighting.disabled = false
    interpolation.disabled = false
    edges.disabled = false
    edges_color.disabled = false
    representation.disabled = false
    frontface_culling.disabled = false
    backface_culling.disabled = false
    ambient.disabled = false
    diffuse.disabled = false
    specular.disabled = false
    specular_power.disabled = false
} else {
    target.renderer_el.getRenderer().removeActor(target.outline_actor)
    opacity.disabled = true
    lighting.disabled = true
    interpolation.disabled = true
    edges.disabled = true
    edges_color.disabled = true
    representation.disabled = true
    frontface_culling.disabled = true
    backface_culling.disabled = true
    ambient.disabled = true
    diffuse.disabled = true
    specular.disabled = true
    specular_power.disabled = true
}
target.renderer_el.getRenderWindow().render()

"""
    },
    args={
        "opacity": opacity,
        "lighting": lighting,
        "interpolation": interpolation,
        "edges": edges,
        "edges_color": edges_color,
        "representation": representation,
        "frontface_culling": frontface_culling,
        "backface_culling": backface_culling,
        "ambient": ambient,
        "diffuse": diffuse,
        "specular": specular,
        "specular_power": specular_power,
    },
)

ui = pn.Column(
    "This example demonstrates the use of **VTK and pyvista** to display a *scene*",
    pn.Row(
        vtkpan.servable(title="VTK - Mt. St Helens"),
        pn.Column(
            actor_selection,
            pn.Tabs(
                (
                    "Scene controller",
                    pn.Column(
                        scene_props,
                        light_props,
                    ),
                ),
                ("Actor properties", actor_props),
            ),
        ).servable(target="sidebar"),
    ),
    min_height=600,
)



import asyncio

from trame.app import asynchronous, get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame app
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2", ws_heart_beat=5)  # 5s
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# State setup
# -----------------------------------------------------------------------------

state.count = 1
state.running_jobs = 0


@asynchronous.task  # <--- if commented the heartbeat will think the server is dead and crash server
async def start_count():
    with state:
        state.count = 0
        state.running_jobs += 1

    for i in range(int(state.delta)):
        with state:
            state.count += 1
            await asyncio.sleep(0.5)

    with state:
        state.running_jobs -= 1


ctrl.run = start_count


# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------
layout = SinglePageLayout(server)

with layout:
    with layout.toolbar as tb:
        vuetify.VSpacer()
        tb.add_child("RunningTasks({{ running_jobs }}) - CountSize({{ delta }})")
        vuetify.VSlider(
            v_model=("delta", 10),
            min=10,
            max=100,
            step=1,
            hide_details=True,
            dense=True,
            classes="mx-4",
        )
        with vuetify.VBtn(icon=True, click=ctrl.run):
            vuetify.VIcon("mdi-run")

    with layout.content:
        with vuetify.VContainer() as container:
            container.add_child("{{ count }}")

# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from pathlib import Path

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import client

CSS_FILE = Path(__file__).with_name("custom.css")

server = get_server()
with DivLayout(server):
    client.Style(CSS_FILE.read_text())


if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.assets.local import LocalFileManager
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

favicons = LocalFileManager(__file__)
favicons.url("open", "./data/fileopen.png")
favicons.url("filter", "./data/filter.png")
favicons.url("kitware", "./data/kw.png")
favicons.url("paraview_mac", "./data/pv_icon.png")
favicons.url("paraview", "./data/pv.png")

_idx = 0
OPTIONS = [*favicons.assets]


def random_update():
    global _idx
    _idx = (_idx + 1) % len(OPTIONS)
    name = OPTIONS[_idx]
    state.favicon = name
    state.trame__title = name


@state.change("favicon")
def update_favicon(favicon, **kwargs):
    state.trame__favicon = favicons[favicon]


# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

layout = SinglePageLayout(server)

with layout:
    layout.title.set_text("Title favicon")

    # Toolbar
    with layout.toolbar as toolbar:
        vuetify.VSpacer()
        vuetify.VSelect(
            label="favicon",
            v_model=("favicon", None),
            items=(
                "favicon_items",
                [{"text": value, "value": value} for value in favicons.assets.keys()],
            ),
            style="max-width: 200px;",
            dense=True,
            hide_details=True,
            classes="mx-4",
        )
        with html.Div(style="width: 200px;", classes="mx-4"):
            vuetify.VTextField(
                label="Title",
                v_model="trame__title",
                dense=True,
                hide_details=True,
            )
        with vuetify.VBtn(icon=True, click=random_update):
            vuetify.VIcon("mdi-refresh")


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk trame-components
"""

from pathlib import Path

from vtkmodules.vtkFiltersCore import vtkThreshold
from vtkmodules.vtkFiltersGeneral import vtkExtractSelectedFrustum
from vtkmodules.vtkIOXML import vtkXMLPolyDataReader
from vtkmodules.web.utils import mesh as vtk_mesh

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, trame, vuetify
from trame.widgets import vtk as vtk_widgets

# -----------------------------------------------------------------------------
# Trame
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Constants
# -----------------------------------------------------------------------------

SCALE_P = 0.0001
SCALE_U = 0.01

VIEW_INTERACT = [
    {"button": 1, "action": "Rotate"},
    {"button": 2, "action": "Pan"},
    {"button": 3, "action": "Zoom", "scrollEnabled": True},
    {"button": 1, "action": "Pan", "alt": True},
    {"button": 1, "action": "Zoom", "control": True},
    {"button": 1, "action": "Pan", "shift": True},
    {"button": 1, "action": "Roll", "alt": True, "shift": True},
]

VIEW_SELECT = [{"button": 1, "action": "Select"}]

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

data_directory = Path(__file__).parent.parent.parent.with_name("data")
f1_vtp = data_directory / "f1.vtp"
# print(f1_vtp)

reader = vtkXMLPolyDataReader()
reader.SetFileName(f1_vtp)
reader.Update()
f1_mesh = reader.GetOutput()
state.f1 = None

# Extract fieldParameters
fieldParameters = {"solid": {"range": [0, 1]}}
pd = f1_mesh.GetPointData()
nb_arrays = pd.GetNumberOfArrays()
for i in range(nb_arrays):
    array = pd.GetArray(i)
    name = array.GetName()
    min, max = array.GetRange(-1)
    fieldParameters[name] = {"name": name, "range": [min, max]}

# Frustrum extraction
extract = vtkExtractSelectedFrustum()
extract.SetInputConnection(reader.GetOutputPort())

threshold = vtkThreshold()
threshold.SetInputConnection(extract.GetOutputPort())
threshold.SetLowerThreshold(0)
threshold.SetInputArrayToProcess(0, 0, 0, 1, "vtkInsidedness")  # 1 => cell


# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

state.trame__title = "F1 Probing"
state.update(
    {
        # Fields available
        "fieldParameters": fieldParameters,
        # picking controls
        "modes": [
            {"value": "hover", "icon": "mdi-magnify"},
            {"value": "click", "icon": "mdi-cursor-default-click-outline"},
            {"value": "select", "icon": "mdi-select-drag"},
        ],
        # Picking feedback
        "pickData": None,
        "selectData": None,
        "tooltip": "",
        "coneVisibility": False,
        "pixel_ratio": 2,
        # Meshes
        "f1Visible": True,
    }
)

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("pickingMode")
def update_picking_mode(pickingMode, **kwargs):
    mode = pickingMode
    if mode is None:
        state.update(
            {
                "tooltip": "",
                "tooltipStyle": {"display": "none"},
                "coneVisibility": False,
                "interactorSettings": VIEW_INTERACT,
            }
        )
    else:
        state.interactorSettings = VIEW_SELECT if mode == "select" else VIEW_INTERACT
        state.update(
            {
                "frustrum": None,
                "selection": None,
                "selectData": None,
            }
        )


@state.change("selectData")
def update_selection(selectData, **kwargs):
    if selectData is None:
        return

    frustrum = selectData.get("frustrum")
    vtk_frustrum = []
    for xyz in frustrum:
        vtk_frustrum += xyz
        vtk_frustrum += [1]

    extract.CreateFrustum(vtk_frustrum)
    extract.ShowBoundsOn()
    extract.PreserveTopologyOff()
    extract.Update()
    state.frustrum = vtk_mesh(extract.GetOutput())
    extract.ShowBoundsOff()
    extract.PreserveTopologyOn()
    threshold.Update()
    state.selection = vtk_mesh(threshold.GetOutput())
    state.selectData = None
    state.pickingMode = None


@state.change("pickData")
def update_tooltip(pickData, pixel_ratio, **kwargs):
    state.tooltip = ""
    state.tooltipStyle = {"display": "none"}
    state.coneVisibility = False
    data = pickData

    if data and data["representationId"] == "f1":
        xyx = data["worldPosition"]
        idx = f1_mesh.FindPoint(xyx)
        if idx > -1:
            messages = []
            cone_state = {
                "resolution": 12,
                "radius": pd.GetArray("p").GetValue(idx) * SCALE_P,
                "center": f1_mesh.GetPoints().GetPoint(idx),
            }

            for i in range(nb_arrays):
                array = pd.GetArray(i)
                name = array.GetName()
                nb_comp = array.GetNumberOfComponents()
                value = array.GetValue(idx)
                value_str = f"{array.GetValue(idx):.2f}"
                norm_str = ""
                if nb_comp == 3:
                    value = array.GetTuple3(idx)
                    norm = (value[0] ** 2 + value[1] ** 2 + value[2] ** 2) ** 0.5
                    norm_str = f" norm({norm:.2f})"
                    value_str = ", ".join([f"{v:.2f}" for v in value])
                    cone_state["height"] = SCALE_U * norm
                    cone_state["direction"] = [v / norm for v in value]

                messages.append(f"{name}: {value_str} {norm_str}")

            if "height" in cone_state:
                new_center = [v for v in cone_state["center"]]
                for i in range(3):
                    new_center[i] -= (
                        0.5 * cone_state["height"] * cone_state["direction"][i]
                    )
                cone_state["center"] = new_center

            if len(messages):
                x, y, z = data["displayPosition"]
                state.coneVisibility = True
                state.tooltip = "\n".join(messages)
                state.cone = cone_state
                state.tooltipStyle = {
                    "position": "absolute",
                    "left": f"{(x / pixel_ratio) + 10}px",
                    "bottom": f"{(y / pixel_ratio) + 10}px",
                    "zIndex": 10,
                    "pointerEvents": "none",
                }


with SinglePageLayout(server) as layout:
    layout.title.set_text("F1 Probing")
    # Let the server know the browser pixel ratio
    trame.ClientTriggers(mounted="pixel_ratio = window.devicePixelRatio")

    with layout.toolbar:
        vuetify.VSpacer()
        with vuetify.VBtnToggle(v_model=("pickingMode", "hover"), dense=True):
            with vuetify.VBtn(value=("item.value",), v_for="item, idx in modes"):
                vuetify.VIcon("{{item.icon}}")
        vuetify.VSelect(
            v_model=("field", "solid"),
            items=(
                "fields",
                [
                    {"value": "solid", "text": "Solid color"},
                    {"value": "p", "text": "Pressure"},
                    {"value": "U", "text": "Velocity"},
                ],
            ),
            classes="ml-8",
            dense=True,
            hide_details=True,
            style="max-width: 140px",
        )
        vuetify.VSelect(
            v_model=("colorMap", "erdc_rainbow_bright"),
            # items=("''|vtkColorPresetItems",),                 # <= For trame-vtk<2.1
            items=("trame.utils.vtk.vtkColorPresetItems('')",),  # <= for trame-vtk>=2.1
            classes="ml-8",
            dense=True,
            hide_details=True,
            style="max-width: 200px",
        )
        vuetify.VSpacer()
        with vuetify.VBtn(icon=True, click="f1Visible = !f1Visible"):
            vuetify.VIcon("mdi-eye-outline", v_if="f1Visible")
            vuetify.VIcon("mdi-eye-off-outline", v_if="!f1Visible")
        with vuetify.VBtn(icon=True, click="$refs.view.resetCamera()"):
            vuetify.VIcon("mdi-crop-free")
        vuetify.VProgressLinear(
            indeterminate=True, absolute=True, bottom=True, active=("trame__busy",)
        )

    with layout.content:
        with vuetify.VContainer(
            fluid=True, classes="pa-0 fill-height", style="position: relative;"
        ):
            with vuetify.VCard(
                style=("tooltipStyle", {"display": "none"}), elevation=2, outlined=True
            ):
                with vuetify.VCardText():
                    html.Pre("{{ tooltip }}")
            with vtk_widgets.VtkView(
                ref="view",
                picking_modes=("[pickingMode]",),
                interactor_settings=("interactorSettings", VIEW_INTERACT),
                click="pickData = $event",
                hover="pickData = $event",
                select="selectData = $event",
            ):
                with vtk_widgets.VtkGeometryRepresentation(
                    id="f1",
                    v_if="f1",
                    color_map_preset=("colorMap",),
                    color_data_range=("fieldParameters[field].range",),
                    actor=("{ visibility: f1Visible }",),
                    mapper=(
                        """{
                            colorByArrayName: field,
                            scalarMode: 3,
                            interpolateScalarsBeforeMapping: true,
                            scalarVisibility: field !== 'solid',
                        }""",
                    ),
                ):
                    vtk_widgets.VtkMesh("f1", dataset=f1_mesh, point_arrays=["p", "U"])
                with vtk_widgets.VtkGeometryRepresentation(
                    id="selection",
                    actor=("{ visibility: !!selection }",),
                    property=(
                        "{ color: [0.99,0.13,0.37], representation: 0, pointSize: Math.round(5 * pixel_ratio)}",
                    ),
                ):
                    vtk_widgets.VtkMesh("selection", state=("selection", None))
                with vtk_widgets.VtkGeometryRepresentation(
                    id="frustrum",
                    actor=("{ visibility: !!frustrum }",),
                ):
                    vtk_widgets.VtkMesh("frustrum", state=("frustrum", None))
                with vtk_widgets.VtkGeometryRepresentation(
                    id="pointer",
                    property=("{ color: [1, 0, 0]}",),
                    actor=("{ visibility: coneVisibility }",),
                ):
                    vtk_widgets.VtkAlgorithm(
                        vtk_class="vtkConeSource",
                        state=("cone", {}),
                    )
# -----------------------------------------------------------------------------
# Jupyter
# -----------------------------------------------------------------------------


def show(**kwargs):
    from trame.app import jupyter

    jupyter.show(server, **kwargs)


# -----------------------------------------------------------------------------
# CLI
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame_server.utils.hot_reload import hot_reload

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# Can do `export TRAME_HOT_RELOAD=1` instead
server.hot_reload = True

# Way to enable widgets when UI creation is deferred after server.start()
html.initialize(server)
vuetify.initialize(server)


# -----------------------------------------------------------------------------
# Dynamically modify any `ChangeMe` to see the new code execute while
# interacting with the app.
# -----------------------------------------------------------------------------
@ctrl.set("number_reset")
def reset_number():
    print("reset_number::ChangeMe")
    state.number = 6
    state.size = 1
    do_someting()


@state.change("number")
def update_number(number, **kwargs):
    print("update_number::ChangeMe", number)
    do_someting()


@hot_reload
def do_someting():
    print("do_someting::ChangeMe")


@ctrl.add("on_server_bind")  # on_server_bind => fn(aiohttp_server)
@ctrl.add("on_server_reload")  # on_server_reload => fn()
def setup_ui(*_):
    with SinglePageLayout(server) as layout:
        layout.title.set_text("Hot Reload - ChangeMe")

        # Toolbar
        with layout.toolbar as toolbar:
            toolbar.dense = True
            vuetify.VSpacer()
            vuetify.VSlider(
                hide_details=True,
                v_model=("number", 6),
                max=60,
                min=3,
                step=1,
                style="max-width: 300px;",
            )
            vuetify.VSlider(
                label="size",
                hide_details=True,
                v_model=("size", 2),
                max=10,
                min=1,
                step=1,
                style="max-width: 300px;",
            )
            with vuetify.VBtn(icon=True, click=ctrl.number_reset):
                vuetify.VIcon("mdi-undo")

        with layout.content:
            with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
                with vuetify.VCol():
                    html.Div(
                        "{{ number }} x {{ i }} = {{ number * i }}",
                        v_for="i in size",
                        key="i",
                        classes="mx-auto text-h6 text-center",
                    )


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from vtkmodules.vtkFiltersSources import vtkConeSource
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk, vuetify

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

DEFAULT_RESOLUTION = 6

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

cone_source = vtkConeSource()
mapper = vtkPolyDataMapper()
actor = vtkActor()
mapper.SetInputConnection(cone_source.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# trame
# -----------------------------------------------------------------------------

server = get_server(
    client_type="vue2",
    # log_network="/Users/sebastien.jourdain/Documents/code/open-source/Web/trame-next/test-examples/logs",
)
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("resolution")
def update_cone(resolution=DEFAULT_RESOLUTION, **kwargs):
    cone_source.SetResolution(resolution)
    ctrl.view_update()


def update_reset_resolution():
    state.resolution = DEFAULT_RESOLUTION


# Bind on life cycle method
ctrl.on_server_ready.add(update_cone)

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

layout = SinglePageLayout(server)
with layout:
    layout.title.set_text("Cone Application")
    layout.icon.click = ctrl.view_reset_camera

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        vuetify.VDivider(vertical=True, classes="mx-2")
        with vuetify.VBtn(icon=True, click=update_reset_resolution):
            vuetify.VIcon("mdi-undo-variant")

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            view = vtk.VtkLocalView(renderWindow, ref="view")
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

from paraview import simple

from trame.app import get_server
from trame.decorators import TrameApp, change
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import (
    paraview as pv_widgets,
)
from trame.widgets import (
    vuetify3 as v3,
)

# -----------------------------------------------------------------------------


@TrameApp()
class ConeApp:
    def __init__(self, server=None):
        # ParaView
        self.cone = simple.Cone()
        self.representation = simple.Show(self.cone)
        self.view = simple.Render()

        # Trame setup
        self.server = get_server(server, client_type="vue3")
        self.state.trame__title = "ParaView - Cone"
        self.ui = self._build_ui()

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    @change("resolution")
    def on_resolution_change(self, resolution, **_):
        self.cone.Resolution = resolution
        self.ctrl.view_update()

    def reset_resolution(self):
        self.state.resolution = 6

    def _build_ui(self):
        with SinglePageLayout(self.server, full_height=True) as layout:
            layout.icon.click = self.ctrl.view_reset_camera
            layout.title.set_text("ParaView - Cone")

            with layout.toolbar:
                v3.VSpacer()
                v3.VSlider(
                    v_model=("resolution", 6),
                    min=3,
                    max=60,
                    step=1,
                    hide_details=True,
                    dense=True,
                    style="max-width: 300px",
                )
                v3.VDivider(vertical=True, classes="mx-2")
                with v3.VBtn(icon=True, click=self.reset_resolution):
                    v3.VIcon("mdi-undo-variant")

            with layout.content:
                with v3.VContainer(fluid=True, classes="pa-0 fill-height"):
                    html_view = pv_widgets.VtkRemoteView(self.view, interactive_ratio=1)
                    self.ctrl.view_reset_camera = html_view.reset_camera
                    self.ctrl.view_update = html_view.update

            return layout


# -----------------------------------------------------------------------------


def main(**kwargs):
    app = ConeApp()
    app.server.start(**kwargs)


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    main()



"""
pip install trame trame-vuetify
python ./495.py
"""

from trame.app import get_server
from trame.ui.vuetify3 import VAppLayout
from trame.widgets import client, html, vuetify3

server = get_server(client_type="vue3")

# 1. Grab the font
server.enable_module(
    {
        "styles": [
            "https://cdn.jsdelivr.net/npm/@tabler/icons-webfont@3.2.0/dist/tabler-icons.min.css"
        ]
    }
)


with VAppLayout(server, full_height=True):
    # 2. Force .ti font-family
    client.Style(".ti:before{font-family:tabler-icons!important}")

    with vuetify3.VContainer(fluid=True, classes="full-height"):
        # MDI
        with vuetify3.VCard(classes="d-flex justify-space-around align-center ma-4"):
            vuetify3.VIcon(
                color="green-darken-2",
                icon="mdi-domain",
                size="x-small",
            )
            vuetify3.VIcon(
                color="blue-darken-2",
                icon="mdi-message-text",
                size="small",
            )
            vuetify3.VIcon(
                color="purple-darken-2",
                icon="mdi-dialpad",
                size="medium",
            )
            vuetify3.VIcon(
                color="teal-darken-2",
                icon="mdi-email",
                size="large",
            )
            vuetify3.VIcon(
                color="blue-grey-darken-2",
                icon="mdi-call-split",
                size="x-large",
            )
            vuetify3.VIcon(
                color="orange-darken-2",
                icon="mdi-arrow-up-bold-box-outline",
                size="large",
            )

        with vuetify3.VCard(classes="d-flex justify-space-around align-center  ma-4"):
            with vuetify3.VBtn("Accept", classes="ma-2", color="primary"):
                vuetify3.VIcon(icon="mdi-checkbox-marked-circle", end=True)
            with vuetify3.VBtn("Decline", classes="ma-2", color="red"):
                vuetify3.VIcon(icon="mdi-cancel", end=True)
            with vuetify3.VBtn(classes="ma-2") as btn:
                vuetify3.VIcon(icon="mdi-minus-circle", start=True)
                btn.add_child("Cancel")

            with vuetify3.VBtn(classes="ma-2", color="orange-darken-2") as btn:
                vuetify3.VIcon(icon="mdi-arrow-left", start=True)
                btn.add_child("Back")

            with html.Div():
                vuetify3.VBtn(classes="ma-2", color="purple", icon="mdi-wrench")
                vuetify3.VBtn(classes="ma-2", color="indigo", icon="mdi-cloud-upload")

            with html.Div():
                vuetify3.VBtn(
                    classes="ma-2",
                    color="blue-lighten-2",
                    icon="mdi-thumb-up",
                    variant="text",
                )
                vuetify3.VBtn(
                    classes="ma-2",
                    color="red-lighten-2",
                    icon="mdi-thumb-down",
                    variant="text",
                )

        # Tabler
        with vuetify3.VCard(classes="d-flex justify-space-around align-center ma-4"):
            vuetify3.VIcon(
                color="green-darken-2",
                icon="ti ti-ghost-2",
                size="x-small",
            )
            vuetify3.VIcon(
                color="blue-darken-2",
                icon="ti ti-ghost-3",
                size="small",
            )
            vuetify3.VIcon(
                color="purple-darken-2",
                icon="ti ti-pacman",
                size="medium",
            )
            vuetify3.VIcon(
                color="teal-darken-2",
                icon="ti ti-layout-sidebar-right",
                size="large",
            )
            vuetify3.VIcon(
                color="blue-grey-darken-2",
                icon="ti ti-layout-sidebar-right-filled",
                size="x-large",
            )
            vuetify3.VIcon(
                color="orange-darken-2",
                icon="ti ti-brand-qq",
                size="large",
            )

        with vuetify3.VCard(classes="d-flex justify-space-around ma-4"):
            with vuetify3.VBtn("Accept", classes="ma-2", color="primary"):
                vuetify3.VIcon(icon="ti ti-ghost", end=True)
            with vuetify3.VBtn("Decline", classes="ma-2", color="red"):
                vuetify3.VIcon(icon="ti ti-ghost-2", end=True)
            with vuetify3.VBtn(classes="ma-2") as btn:
                vuetify3.VIcon(icon="ti ti-arrow-roundabout-right", start=True)
                btn.add_child("Cancel")

            with vuetify3.VBtn(classes="ma-2", color="orange-darken-2") as btn:
                vuetify3.VIcon(icon="ti ti-arrow-roundabout-right", start=True)
                btn.add_child("Back")

            with html.Div():
                vuetify3.VBtn(
                    classes="ma-2", color="purple", icon="ti ti-wrecking-ball"
                )
                vuetify3.VBtn(classes="ma-2", color="indigo", icon="ti ti-cloud-upload")

            with html.Div():
                vuetify3.VBtn(
                    classes="ma-2",
                    color="blue-lighten-2",
                    icon="ti ti-thumb-up",
                    variant="text",
                )
                vuetify3.VBtn(
                    classes="ma-2",
                    color="red-lighten-2",
                    icon="ti ti-thumb-down",
                    variant="text",
                )

server.start()



from trame.app import TrameApp
from trame.decorators import change, controller, life_cycle, trigger
from trame.ui.html import DivLayout
from trame.widgets import html


class App(TrameApp):
    def __init__(self, name=None):
        super().__init__(server=name)
        self._build_ui()

    @trigger("exec")
    def method_call(self, msg):
        print("method_called", msg)

    @controller.set("hello")
    def method_on_ctrl(self, *args):
        print("method_on_ctrl", args)

    @change("resolution")
    def one_slider(self, resolution, **kwargs):
        print("Slider value 1", resolution)

    @life_cycle.server_ready
    def on_ready(self, *args, **kwargs):
        print("on_ready")

    def _build_ui(self):
        with DivLayout(self.server) as self.ui:
            html.Input(
                type="range",
                min=3,
                max=60,
                step=1,
                v_model_number=("resolution", 6),
            )
            html.Button("trigger", click="trigger('exec', ['trigger'])")
            html.Button("method", click=(self.method_call, "['method']"))
            html.Button("ctrl", click=self.ctrl.hello)


if __name__ == "__main__":
    app = App()
    app.server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/PlainPython/GeoMaps/MappingDemo/app.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/c7aba7c09c4b0d29cf1e4015ae65f214f1828805

Installation requirements:
    pip install trame trame-vuetify trame-vtk trame-deckgl
"""

import os

import pandas as pd
import pydeck as pdk

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import deckgl, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Getting a Mapbox API key
# -----------------------------------------------------------------------------
# By default, pydeck 0.6 provides basemap tiles through Carto.
#
# You can optionally use a Mapbox API key, by registering for Mapbox via
# this link [1]. You should then create a new public API token [2].
# You can learn more about Mapbox tokens via their documentation [3].
#
# [1] https://account.mapbox.com/auth/signup/
# [2] https://account.mapbox.com/access-tokens/
# [3] https://docs.mapbox.com/help/how-mapbox-works/access-tokens/#how-access-tokens-work
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Deck.gl / MapBox
# Expect MAPBOX_API_KEY environment variable
# -----------------------------------------------------------------------------

defaultLayers = [
    "Bike Rentals",
    "Bart Stop Exits",
    "Bart Stop Names",
    "Outbound Flow",
]

# -----------------------------------------------------------------------------


def from_data_file(filename):
    url = (
        "https://raw.githubusercontent.com/streamlit/"
        "example-data/master/hello/v1/%s" % filename
    )
    return pd.read_json(url)


ALL_LAYERS = {
    "Bike Rentals": pdk.Layer(
        "HexagonLayer",
        data=from_data_file("bike_rental_stats.json"),
        get_position=["lon", "lat"],
        radius=200,
        elevation_scale=4,
        elevation_range=[0, 1000],
        extruded=True,
    ),
    "Bart Stop Exits": pdk.Layer(
        "ScatterplotLayer",
        data=from_data_file("bart_stop_stats.json"),
        get_position=["lon", "lat"],
        get_color=[200, 30, 0, 160],
        get_radius="[exits]",
        radius_scale=0.05,
    ),
    "Bart Stop Names": pdk.Layer(
        "TextLayer",
        data=from_data_file("bart_stop_stats.json"),
        get_position=["lon", "lat"],
        get_text="name",
        get_color=[0, 0, 0, 200],
        get_size=15,
        get_alignment_baseline="'bottom'",
    ),
    "Outbound Flow": pdk.Layer(
        "ArcLayer",
        data=from_data_file("bart_path_stats.json"),
        get_source_position=["lon", "lat"],
        get_target_position=["lon2", "lat2"],
        get_source_color=[200, 30, 0, 160],
        get_target_color=[200, 30, 0, 160],
        auto_highlight=True,
        width_scale=0.0001,
        get_width="outbound",
        width_min_pixels=3,
        width_max_pixels=30,
    ),
}

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("activeLayers")
def update_map(activeLayers, **kwargs):
    selected_layers = [
        layer for layer_name, layer in ALL_LAYERS.items() if layer_name in activeLayers
    ]

    if selected_layers:
        deck = pdk.Deck(
            map_provider="mapbox",
            map_style="mapbox://styles/mapbox/light-v9",
            initial_view_state={
                "latitude": 37.76,
                "longitude": -122.4,
                "zoom": 11,
                "pitch": 50,
            },
            layers=selected_layers,
        )
        ctrl.deck_update(deck)
    else:
        state.error = "Please choose at least one layer above."


# -----------------------------------------------------------------------------
# GUI Layout
# -----------------------------------------------------------------------------

state.trame__title = "Deck + Mapbox Demo"
with SinglePageLayout(server) as layout:
    layout.title.set_text("Deck + Mapbox Demo")

    with layout.content:
        deckMap = deckgl.Deck(
            mapbox_api_key=os.environ["MAPBOX_API_KEY"],
            style="width: 100vw;",
            classes="fill-height",
        )
        ctrl.deck_update = deckMap.update
        vuetify.VSelect(
            style="position: absolute; top: 10px; left: 25px; width: 600px;",
            items=("layerNames", defaultLayers),
            v_model=("activeLayers", defaultLayers),
            dense=True,
            hide_details=True,
            multiple=True,
            chips=True,
        )

# -----------------------------------------------------------------------------
# Start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



# Configuration file for the Sphinx documentation builder.

import inspect
import os
import re
from pathlib import Path

from sphinx.ext import apidoc

import trame

HTML_ELEMENT_REGEX = re.compile(r"^trame.*\.widgets\..*HtmlElement$")

# -- Theme configuration -----------------------------------------------------

# html_theme_options = {
#     'analytics_id': 'G-XXXXXXXXXX',  #  Provided by Google in your dashboard
#     'analytics_anonymize_ip': False,
#     'logo_only': False,
#     'prev_next_buttons_location': 'bottom',
#     'style_external_links': False,
#     'vcs_pageview_mode': '',
#     'style_nav_header_background': 'white',
#     'flyout_display': 'hidden',
#     'version_selector': True,
#     'language_selector': True,
#     # Toc options
#     'collapse_navigation': True,
#     'sticky_navigation': True,
#     'navigation_depth': 4,
#     'includehidden': True,
#     'titles_only': False
# }

# -- Project information -----------------------------------------------------

project = "Trame"
copyright = "2022, Kitware"
author = "Kitware"

# -- General configuration ---------------------------------------------------

extensions = [
    "sphinx.ext.duration",
    "sphinx.ext.doctest",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.intersphinx",
]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "sphinx": ("https://www.sphinx-doc.org/en/master/", None),
}
intersphinx_disabled_domains = ["std"]

templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for autodoc -----------------------------------------------------
autodoc_member_order = "bysource"
autodoc_mock_imports = [
    "IPython",
    "mpld3",
]

# -- Options for HTML output -------------------------------------------------

html_theme = "sphinx_rtd_theme"

html_static_path = ["_static"]

html_css_files = [
    "css/custom.css",
]

# -- Modify environment variables --------------------------------------------

os.environ.update(
    {
        "TRAME_PARAVIEW_FAIL_SILENTLY": "True",
    }
)

# -- Hooks for sphinx events -------------------------------------------------


def run_apidoc(_):
    # Override the apidoc options with what we want
    apidoc.OPTIONS.clear()
    apidoc.OPTIONS.extend(
        [
            "members",
            "imported-members",
            "show-inheritance",
        ]
    )

    exclude_paths = [
        "env/utils.py",
    ]

    cur_path = str(Path(__file__).parent)
    templates_path = str(Path(cur_path) / "apidoc_templates")
    module_path = str(Path(trame.__file__).parent)

    # Make the exclude paths absolute
    exclude_paths = [str(Path(module_path) / x) for x in exclude_paths]

    argv = [
        "-f",
        "-T",
        "-e",
        "-M",
        "-o",
        cur_path,
        "-t",
        templates_path,
        module_path,
    ] + exclude_paths

    apidoc.main(argv)


def maybe_skip_member(app, what, name, obj, skip, options):
    # Implement logic here for whether to skip certain members
    return skip


def autodoc_process_bases(app, name, obj, options, bases):
    if len(bases) == 1 and inspect.isclass(bases[0]):
        cls = bases[0]
        name_with_module = f"{cls.__module__}.{cls.__qualname__}"
        if HTML_ELEMENT_REGEX.match(name_with_module):
            # It is an HtmlElement. Use it's base class instead.
            bases[0] = cls.__bases__[0]


def setup(app):
    app.connect("builder-inited", run_apidoc)
    app.connect("autodoc-skip-member", maybe_skip_member)
    app.connect("autodoc-process-bases", autodoc_process_bases)



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk trame-components
"""

from vtkmodules.vtkIOXML import vtkXMLPolyDataReader
from vtkmodules.web.utils import mesh as vtk_mesh

from trame.app import get_server
from trame.app.file_upload import ClientFile
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import trame, vuetify
from trame.widgets import vtk as vtk_widgets

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("files")
def load_client_files(files, **kwargs):
    if files is None or len(files) == 0:
        return

    field = "solid"
    fields = {
        "solid": {"value": "solid", "text": "Solid color", "range": [0, 1]},
    }
    meshes = []
    filesOutput = []

    if files and len(files):
        if not files[0].get("content"):
            return

        for file in files:
            file = ClientFile(file)
            print(f"Load {file.name}")
            bytes = file.content
            filesOutput.append({"name": file.name, "size": file.size})
            reader = vtkXMLPolyDataReader()
            reader.ReadFromInputStringOn()
            reader.SetInputString(bytes)
            reader.Update()
            ds = reader.GetOutputAsDataSet(0)
            point_arrays = []
            pd = ds.GetPointData()
            nb_arrays = pd.GetNumberOfArrays()
            for i in range(nb_arrays):
                array = pd.GetArray(i)
                name = array.GetName()
                min, max = array.GetRange(-1)
                fields[name] = {
                    "name": name,
                    "range": [min, max],
                    "value": name,
                    "text": name,
                    "scalarMode": 3,
                }
                point_arrays.append(name)

            cell_arrays = []
            cd = ds.GetCellData()
            nb_arrays = cd.GetNumberOfArrays()
            for i in range(nb_arrays):
                array = cd.GetArray(i)
                name = array.GetName()
                min, max = array.GetRange(-1)
                fields[name] = {
                    "name": name,
                    "range": [min, max],
                    "value": name,
                    "text": name,
                    "scalarMode": 4,
                }
                cell_arrays.append(name)

            meshes.append(
                vtk_mesh(ds, point_arrays=point_arrays, cell_arrays=cell_arrays)
            )

    state.field = field
    state.fields = fields
    state.meshes = meshes
    state.files = filesOutput
    print(f"show {len(meshes)} meshes")


# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

state.trame__title = "File loading"
state.fields = []
state.meshes = []

with SinglePageLayout(server) as layout:
    layout.title.set_text("File Loader")
    layout.icon.click = ctrl.view_reset_camera
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSelect(
            v_model=("field", "solid"),
            items=("Object.values(fields)",),
            hide_details=True,
            dense=True,
            style="max-width: 200px;",
            classes="mr-4",
        )
        vuetify.VFileInput(
            multiple=True,
            show_size=True,
            small_chips=True,
            truncate_length=25,
            v_model=("files", None),
            dense=True,
            hide_details=True,
            style="max-width: 300px;",
            accept=".vtp",
            __properties=["accept"],
        )
        vuetify.VProgressLinear(
            indeterminate=True, absolute=True, bottom=True, active=("trame__busy",)
        )
        trame.ClientStateChange(name="meshes", change=ctrl.view_reset_camera)

    with layout.content:
        with vuetify.VContainer(
            fluid=True, classes="pa-0 fill-height", style="position: relative;"
        ):
            with vtk_widgets.VtkView(ref="view") as view:
                ctrl.view_reset_camera = view.reset_camera
                with vtk_widgets.VtkGeometryRepresentation(
                    v_for="mesh, idx in meshes",
                    key=("idx",),
                    color_data_range=(
                        "fields[field] && fields[field].range || [0, 1]",
                    ),
                    mapper=(
                        """{
                            colorByArrayName: field,
                            scalarMode: fields[field] && fields[field].scalarMode || 3,
                            interpolateScalarsBeforeMapping: true,
                            scalarVisibility: field !== 'solid',
                        }""",
                    ),
                ):
                    vtk_widgets.VtkMesh("myMesh", state=("mesh",))


# -----------------------------------------------------------------------------
# CLI
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.visibilityList = [True, False, True, False]


def update_visibility(index, visibility):
    state.visibilityList[index] = visibility
    state.dirty("visibilityList")
    print(f"Toggle {index} to {visibility}")


with SinglePageLayout(server) as layout:
    with layout.content:
        with vuetify.VContainer(fluid=True, classes="fill-height pa-0 ma-0"):
            with vuetify.VCol():
                vuetify.VCheckbox(
                    v_for="v, i in visibilityList",
                    key="i",
                    label=("`Hello ${i}`",),
                    v_model=("visibilityList[i]",),
                    change=(update_visibility, "[i, $event]"),
                )


if __name__ == "__main__":
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/SimpleCone/LocalRendering.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/674f72774228bbcab5689417c1c5642230b1eab8

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

from vtkmodules.vtkFiltersSources import vtkConeSource
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "VTK Remote View - Local Rendering"

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

DEFAULT_RESOLUTION = 6

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

cone_source = vtkConeSource()
mapper = vtkPolyDataMapper()
actor = vtkActor()
mapper.SetInputConnection(cone_source.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("resolution")
def update_cone(resolution=DEFAULT_RESOLUTION, **kwargs):
    cone_source.SetResolution(resolution)
    ctrl.view_update()


def update_reset_resolution():
    state.resolution = DEFAULT_RESOLUTION


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("Cone Application")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        vuetify.VDivider(vertical=True, classes="mx-2")
        with vuetify.VBtn(icon=True, click=update_reset_resolution):
            vuetify.VIcon("mdi-undo-variant")

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            view = vtk_widgets.VtkLocalView(renderWindow, ref="view")
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera

# -----------------------------------------------------------------------------
# Jupyter
# -----------------------------------------------------------------------------


def show(**kwargs):
    from trame.app import jupyter

    jupyter.show(server, **kwargs)


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame_common.assets.remote import (
    AbstractRemoteFile,
    GoogleDriveFile,
    HttpFile,
    download_file_from_google_drive,
)

__all__ = [
    "AbstractRemoteFile",
    "GoogleDriveFile",
    "HttpFile",
    "download_file_from_google_drive",
]



import vtk
import vtk.util.numpy_support

# trame imports
import trame.app
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify


def get_scalar_bar(colorTransferFunction):
    scalar_bar = vtk.vtkScalarBarActor()
    scalar_bar.SetLookupTable(colorTransferFunction)
    scalar_bar.SetTitle("Color Temp")
    scalar_bar.UnconstrainedFontSizeOn()
    scalar_bar.SetNumberOfLabels(5)
    scalar_bar.SetMaximumWidthInPixels(800 // 8)
    scalar_bar.SetMaximumHeightInPixels(800 // 3)
    scalar_bar.SetObjectName("ScalarBar")
    return scalar_bar


ren1 = vtk.vtkRenderer()
colorTransferFunction = vtk.vtkColorTransferFunction()
colorTransferFunction.AddRGBPoint(0.0, 0.69, 0.69, 0.69)
colorTransferFunction.AddRGBPoint(1.0, 1.0, 0.3, 0.3)
scalar_bar = get_scalar_bar(colorTransferFunction)
ren1.AddActor2D(scalar_bar)
renderWindow = vtk.vtkRenderWindow()
renderWindow.AddRenderer(ren1)
renderWindow.Render()


server = trame.app.get_server(name="1234", client_type="vue2")
state, ctrl = server.state, server.controller

with SinglePageLayout(server) as layout:
    layout.title.set_text("2D View")
    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            view = vtk_widgets.VtkLocalView(renderWindow, ref="view")
            ctrl.view_update = view.update

server.start()



# pip install psutil watchdog

import argparse
import asyncio
import os
import pwd
from functools import partial
from pathlib import Path

import psutil
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer


def cli():
    parser = argparse.ArgumentParser(
        prog="Trame session monitoring",
        description="This application will monitor Python processes listening to network",
    )
    parser.add_argument(
        "--port-range",
        nargs=2,
        help="Inclusive port range to check",
        type=int,
        default=[9001, 9500],
    )
    parser.add_argument(
        "--user", help="User name", default=pwd.getpwuid(os.getuid())[0]
    )
    parser.add_argument("--watch", help="File to watch for new sessions")
    parser.add_argument(
        "--target",
        help="Directory to fill with empty file matching active port",
        required=True,
    )
    parser.add_argument(
        "--refresh",
        help="Time in second for refresh",
        type=int,
        default=60,  # 60 seconds
    )
    options, _ = parser.parse_known_args()
    return options


def update_running(user, port_range, target_directory):
    # Gather current sessions
    target_directory = Path(target_directory)
    current_ports = set()
    for f in target_directory.iterdir():
        if f.is_file():
            try:
                port = int(f.name)
                current_ports.add(port)
            except ValueError:
                pass

    # Check active processes + port
    file_to_delete = set(current_ports)
    for pid in psutil.pids():
        try:
            p = psutil.Process(pid)
        except psutil.NoSuchProcess:
            continue

        if p.username() == user and "python" in p.name():
            conns = p.net_connections(kind="tcp")
            for con in conns:
                port = con.laddr.port
                if port_range[0] <= port <= port_range[1]:
                    file_to_delete.discard(port)
                    (target_directory / f"{port}").touch()

    # Cleaning up dead sessions
    for port in file_to_delete:
        (target_directory / f"{port}").unlink()


async def run_forever(fn, delta_t):
    while True:
        fn()
        await asyncio.sleep(delta_t)


class ExecOnChange(FileSystemEventHandler):
    def __init__(self, fn, file_path):
        super().__init__()
        self._fn = fn
        self._file_path = file_path

    def on_modified(self, event):
        if event.src_path == self._file_path:
            self._fn()


def main():
    args = cli()
    update = partial(
        update_running,
        user=args.user,
        port_range=args.port_range,
        target_directory=args.target,
    )

    print("- target directory:", args.target)
    print("- user:", args.user)
    print("- port range:", args.port_range)
    print("- refresh:", args.refresh)

    if args.watch and Path(args.watch).exists():
        watch_file = Path(args.watch).resolve()
        print("- watch:", watch_file)

        observer = Observer()
        observer.schedule(
            ExecOnChange(update, str(watch_file)),
            str(watch_file.parent),
            recursive=False,
        )
        observer.start()

    asyncio.run(run_forever(update, args.refresh))


if __name__ == "__main__":
    main()



from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import vuetify3 as vuetify

server = get_server(client_type="vue3")
state, ctrl = server.state, server.controller

# Adding missing font
server.enable_module(
    {"styles": ["https://fonts.googleapis.com/css?family=Roboto:300,400,500"]}
)

with SinglePageLayout(server) as layout:
    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="d-flex justify-center align-center",
            style="height: 100%;",
        ):
            with vuetify.VCard():
                with vuetify.VCardTitle(
                    "Centered Card",
                    classes="d-flex align-center flex-wrap",
                    # style="font-weight: 500 !important"
                ):
                    vuetify.VSpacer()
                    vuetify.VBtn(icon="mdi-refresh", density="compact")
                vuetify.VDivider()
                with vuetify.VCardText():
                    vuetify.VTextField(
                        v_model=("text_value", ""),
                        label="Some text",
                        placeholder="Type here...",
                        variant="outlined",
                    )

    # print(layout)

server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-router
"""

from trame.app import get_server
from trame.ui.router import RouterViewLayout
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import router, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

# Home route
with RouterViewLayout(server, "/"):
    with vuetify.VCard():
        vuetify.VCardTitle("This is home")

# Foo route
with RouterViewLayout(server, "/foo"):
    with vuetify.VCard():
        vuetify.VCardTitle("This is foo")
        with vuetify.VCardText():
            vuetify.VBtn("Take me back", click="$router.back()")

# Bar/id
with RouterViewLayout(server, "/bar/:id"):
    with vuetify.VCard():
        vuetify.VCardTitle("This is bar with ID '{{ $route.params.id }}'")

# Main page content
with SinglePageWithDrawerLayout(server) as layout:
    layout.title.set_text("Multi-Page demo")

    with layout.content:
        with vuetify.VContainer():
            router.RouterView()

    # add router buttons to the drawer
    with layout.drawer:
        with vuetify.VList(shaped=True, v_model=("selectedRoute", 0)):
            vuetify.VSubheader("Routes")

            with vuetify.VListItem(to="/"):
                with vuetify.VListItemIcon():
                    vuetify.VIcon("mdi-home")
                with vuetify.VListItemContent():
                    vuetify.VListItemTitle("Home")

            with vuetify.VListItem(to="/foo"):
                with vuetify.VListItemIcon():
                    vuetify.VIcon("mdi-food")
                with vuetify.VListItemContent():
                    vuetify.VListItemTitle("Foo")

            with vuetify.VListGroup(value=("true",), sub_group=True):
                with vuetify.Template(v_slot_activator=True):
                    vuetify.VListItemTitle("Bars")
                with vuetify.VListItemContent():
                    with vuetify.VListItem(v_for="id in [1,2,3]", to=("'/bar/' + id",)):
                        with vuetify.VListItemIcon():
                            vuetify.VIcon("mdi-peanut-outline")
                        with vuetify.VListItemContent():
                            vuetify.VListItemTitle("Bar")
                            vuetify.VListItemSubtitle("ID '{{id}}'")

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

# import paraview.web.venv  # Available in PV 5.11
from paraview import simple

from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import paraview, vuetify3

# -----------------------------------------------------------------------------

server = get_server(client_type="vue3")
state, ctrl = server.state, server.controller
state.trame__title = "ParaView cone"

# -----------------------------------------------------------------------------
# ParaView code
# -----------------------------------------------------------------------------

DEFAULT_RESOLUTION = 6

cone = simple.Cone()
representation = simple.Show(cone)
view = simple.Render()


@state.change("resolution")
def update_cone(resolution, **kwargs):
    cone.Resolution = resolution
    ctrl.view_update()


def update_reset_resolution():
    state.resolution = DEFAULT_RESOLUTION


# -----------------------------------------------------------------------------


with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("Cone Application")

    with layout.toolbar:
        vuetify3.VSpacer()
        vuetify3.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            density="compact",
            style="max-width: 300px",
        )
        vuetify3.VDivider(vertical=True, classes="mx-2")
        vuetify3.VBtn(icon="mdi-undo-variant", click=update_reset_resolution)

    with layout.content:
        with vuetify3.VContainer(fluid=True, classes="pa-0 fill-height"):
            html_view = paraview.VtkLocalView(view)
            ctrl.view_reset_camera = html_view.reset_camera
            ctrl.view_update = html_view.update

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify
"""

from trame.app import get_server
from trame.ui.vuetify import VAppLayout
from trame.widgets import vuetify

server = get_server(client_type="vue2")

with VAppLayout(server):
    vuetify.VBtn(
        "Download",
        click="utils.download('my_file_name.csv', file_content, 'text/csv')",
    )

server.state.file_content = """a,b,c
1,2,3
4,5,6
7,8,9
"""


if __name__ == "__main__":
    server.start()
